// GameState.cpp: implementation of the CGameState class.
//
//////////////////////////////////////////////////////////////////////

#include "Adventure.h"
#include "CGameState.h"
// #include <direct.h>			// needed for mkdir();
#include <sys/stat.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGameState::CGameState() : m_locVisitedCnt(150,50), m_locCondition(150,50), m_hintAreaCnt(16,16),
						 m_objectLoc1(100), m_objectLoc2(100), m_objectProperty(100),
						 m_dwarfLoc1(DWARFMAX), m_dwarfSeen(DWARFMAX), m_dwarfOldLoc(DWARFMAX)
{
}


CGameState::~CGameState()
{
										
}

/*
		Utility Routines
*/

/*
		display current brief status.
*/
bool CGameState::Get_brief( bool bDisplay )
{
	bool bRc = m_brief != 0;

	if ( m_brief < 0 )
		m_brief = 0;
	if ( m_brief > 1 )
		m_brief = 1;
							
	if ( bDisplay )
		SpeakInfoMsg(MSG4_220_BriefMessageOff+m_brief);
							
	return ( bRc ); 
}

bool CGameState::Set_brief( short brief, bool bDisplay )
{
	m_brief = brief;
	return Get_brief( bDisplay );
}

bool CGameState::Toggle_brief( bool bDisplay )
{
	m_brief ^= 1;	// toggle brief
	return Get_brief( bDisplay ); 
}

short CGameState::Get_Friendly( bool bDisplay )
{
	if ( m_nFriendly < 0 )
		m_nFriendly = 0;
	if ( m_nFriendly > 1 )
		m_nFriendly = 1;

	if ( bDisplay )
		SpeakInfoMsg(MSG4_222_FriendlyOff+m_nFriendly);
	return m_nFriendly;
}

short CGameState::Toggle_Friendly( bool bDisplay )
{
	m_nFriendly ^= 1;
	return Get_Friendly(bDisplay);
}

short CGameState::Get_VocabCmpLen( bool bDisplay )
{
	if ( bDisplay )
	{
		if ( m_nVocabCmpLen == 5)
			SpeakInfoMsg(MSG4_231_VocabLen5);
		else if ( m_nVocabCmpLen == 20)
			SpeakInfoMsg(MSG4_232_VocabLen20);
		else
		{
			printf("vocabulary search length is now %d\n", m_nVocabCmpLen);
			puts("Say \"vocab\" to switch to exact matching.\n");
		}
	}
	return m_nVocabCmpLen;
}

short CGameState::Toggle_VocabCmpLen( bool bDisplay )
{
	m_nVocabCmpLen = m_nVocabCmpLen == 20 ? 5 : 20;
	return Get_VocabCmpLen(bDisplay);
}

short CGameState::Get_LogLevel( bool bDisplay )
{
	if ( bDisplay )
		SpeakInfoMsg(MSG4_224_LogToFileOff+m_nLogLevel);
	return m_nLogLevel;
}

short CGameState::Toggle_LogLevel( bool bDisplay )
{
	if ( ++m_nLogLevel > 2 )
		m_nLogLevel = 0;
	return Get_LogLevel(bDisplay);
}

short CGameState::Get_OfferHints( bool bDisplay )
{
	if ( m_nOfferHints < 0 )
		m_nOfferHints = 0;
	if ( m_nOfferHints > 1 )
		m_nOfferHints = 1;

	if ( bDisplay )
		SpeakInfoMsg(MSG4_233_OfferHintsOff+m_nOfferHints);
	return m_nOfferHints;
}

short CGameState::Toggle_OfferHints( bool bDisplay )
{
	m_nOfferHints ^= 1;
	return Get_OfferHints(bDisplay);
}

short CGameState::Get_bAutoSave( bool bDisplay )
{
	if ( m_bAutoSave < 0 )
		m_bAutoSave = 0;
	if ( m_bAutoSave > 1 )
		m_bAutoSave = 1;

	if ( bDisplay )
		SpeakInfoMsg(MSG4_235_AutoSaveOff+m_bAutoSave);
	return m_bAutoSave;
}

short CGameState::Toggle_bAutoSave( bool bDisplay )
{
	m_bAutoSave ^= 1;
	return Get_bAutoSave(bDisplay);
}

void  CGameState::Set_PirateEnabled( short bPirateEnabled)
{
	m_bPirateEnabled = bPirateEnabled;
	if ( m_bPirateEnabled == 0 )
		if ( Get_objectLoc1( OBJ_CHEST ) == LOC_000_Limbo )
			gameState.move( OBJ_CHEST, Get_chestLoc1(), LOC_000_MOVEABLE );
}


/*
		Routine to save and restore stimer vars.
*/
void CGameState::SaveRestoreTime( bool bSave )
{
	static short	tally1;			/* item counts		*/
	static short	tally2;			/* item counts		*/
	static short	limit;			/* time m_limit		*/
	static short	wzdark;			/* game state flags	*/
	static short	closing;		/* game state flags	*/
	static short	closed;			/* game state flags	*/
	static short	clock1;			/* timing variables	*/
	static short	clock2;			/* timing variables	*/
	static short	panic;			/* timing variables	*/
	static short	lampWarning;	/* lamp warning flag*/
	static short	foobar;			/* fie fie foe foo...*/
										
	static bool bFirst = true;
										
	if ( bSave || bFirst )
	{
		tally1		= m_tally1;	
		tally2		= m_tally2;	
		limit		= m_limit;	
		wzdark		= m_wasDark;	
		closing		= m_closing;	
		closed		= m_closed;	
		clock1		= m_clock1;	
		clock2		= m_clock2;	
		panic		= m_panic;	
		lampWarning	= lampWarning;	
		foobar		= m_foobar;
																				
		bFirst	= false;
	}
	else if ( ! m_closed )  // else restore
	{
		m_tally1		= tally1;	
		m_tally2		= tally2;	
		m_limit			= limit;	
		m_wasDark		= wzdark;	
		m_closing		= closing;	
		m_closed		= closed;	
		m_clock1		= clock1;	
		m_clock2		= clock2;	
		m_panic			= panic;	
		m_lampWarning	= lampWarning;
		m_foobar		= foobar;
	}
}


/*
		Routine to test for darkness
*/
short CGameState::dark()
{
	if ( ! ValidCaveLocation( m_location ) )
		return DARK;
	return(!(m_locCondition[m_location] & LIGHT) &&
	(!m_objectProperty[OBJ_LAMP] ||
	!here(OBJ_LAMP)));
}

/*
		Routine to tell if an item is present.
*/
short CGameState::here(short item)
{
	return(m_objectLoc1[item] == m_location || toting(item));
}

/*
		Routine to tell if an item is being carried.
*/
short CGameState::toting(short item)
{
	return(m_objectLoc1[item] == LOC_Neg1_PLAYER);
}

/*
		Routine to tell if a location causes
		a forced move.
*/
short CGameState::forced(short atloc)
{
	if ( ! ValidCaveLocation( atloc ) )
		return 0;

	return(m_locCondition[atloc] & FORCED_MOVE);
}

/*
		Map starts in the tree house.
		Routine to Place the Cave Map in the forrest
		after the player wanders around a while
		and has taken all the items	from the well house
		but hasen't found the cave or the map.
*/
bool CGameState::ShowMapFirst( bool bForced )
{
	if ( m_objectLoc1[OBJ_MAP] != LOC_142_tree_house )
		return false;
	if ( m_objectProperty[OBJ_MAP] > 0 )
		return false;

	if (	m_dwarfFlag > 0
		||	m_locVisitedCnt[LOC_009_below_grate] > 0
		||	m_locVisitedCnt[LOC_011_debris_room] > 0
		||	m_locVisitedCnt[LOC_033_Y2] > 0
		)
		return false;
	
	short	visitCntr = 0;
	short	visitCntrMin = LOC_008_outside_grate * 2;
	short	numObjsIn3 = 0;
	short	mapLoc = bForced ? LOC_003_well_house : LOC_005_forest;
	
	int ii = 0;
	
	for ( ii = LOC_001_end_road; ii < LOC_008_outside_grate; ++ii )
	{
		visitCntr += m_locVisitedCnt[ii];
	}
	visitCntr += m_locVisitedCnt[LOC_141_forest];
	visitCntr += m_locVisitedCnt[LOC_143_main_office];

	for ( ii = 1; ii <= LastObjectIndexed(); ++ii )
	{
		if ( m_objectLoc1[ii] == LOC_003_well_house )
			++numObjsIn3;
	}
	if ( bForced || (visitCntr > visitCntrMin && numObjsIn3 == 0 && mapLoc != m_location ) )
	{
		m_objectLoc1[OBJ_MAP] = mapLoc;
		m_objectLoc2[OBJ_MAP] = LOC_000_MOVEABLE; // getable/carryable .. not fixed
		m_objectProperty[OBJ_MAP] = 0;
		++m_holdMax;
	}
	return m_objectLoc1[OBJ_MAP] > LOC_000_Limbo;
}

/*
		Routine to tell if player is on
		either side of a two sided object.
*/
short CGameState::at(short item)
{
	return(m_objectLoc1[item] == m_location || m_objectLoc2[item] == m_location);
}

/*
		Routine to destroy an object
*/
void CGameState::dstroy(short object)
{
	move(object, LOC_000_Limbo, LOC_000_MOVEABLE);
}

/*
		Juggle an object
		
		C  JUGGLE AN OBJECT BY PICKING IT UP AND PUTTING IT DOWN AGAIN, THE PURPOSE
		C  BEING TO GET THE OBJECT TO THE FRONT OF THE CHAIN OF THINGS AT ITS LOC.
*/
void CGameState::juggle(short object)
{
#if 0								// correct but necessary because we don't use a link.
	int loc1, loc2;
										
	loc1 = m_objectLoc1[object];
	loc2 = m_objectLoc2[object];
	move( object, loc1, loc2 );
#endif
	
}


/*
		routine to move an object and return a
		value used to set the negated m_objectProperty values
		for the repository.
*/
short CGameState::put(short object, short where, short pval)
{
	if ( object < 1 || object > LastObjectIndexed() )
	{
		printf("put(%d) object < 1 || object > LastObjectIndexed()\n", object);
		bug(BUG_GameState_cpp+99);
		return -1;
	}
										
	move(object, where, LOC_Neg2_NO_CHANGE);
	return((-1)-pval);
}


/*
		Routine to move an object

		object is the normal object number to move.
		where1 refers to m_objectLoc1
		where2 refers to m_objectLoc2, used for fixed objects.
*/
void CGameState::move(short object, short where1, short where2)
{
	short fromLoc = 0;
										
	if (   where1 < LOC_Neg1_FIXED
		|| where2 < LOC_Neg2_NO_CHANGE
		// || where >= MAXLOC
		)
	{
		printf("move(%d, %d, %d) bad where\n", object, where1, where2);
		bug(BUG_GameState_cpp+101);
		return;
	}
										
	if ( object < 1 || object > LastObjectIndexed() )
	{
		printf("move(%d) object < 1 || object > LastObjectIndexed()\n", object);
		bug(BUG_GameState_cpp+121);  // invalid object
		return;
	}

	fromLoc = m_objectLoc1[object];

	if ( fromLoc != where1 )
	{
		if ( where1 == LOC_Neg1_PLAYER )
			carry( object, fromLoc );

		if ( fromLoc == LOC_Neg1_PLAYER )
			drop( object, where1 );
		else
			m_objectLoc1[object] = where1;
	}

	if ( where2 != LOC_Neg2_NO_CHANGE )
		m_objectLoc2[object] = where2;	// fixed or moveable
}


/*
		Routine to carry an object
*/
void CGameState::carry(short object, short where)
{
	if ( object < 1 || object > LastObjectIndexed() )
	{
		printf("carry(%d, %d) object < 1 || object > LastObjectIndexed()\n", object, where);
		bug(BUG_GameState_cpp+121);  // invalid object
		return;
	}
										
	if ( m_objectLoc1[object] == LOC_Neg1_PLAYER)
		return;	// already carrying

	m_objectLoc1[object] = LOC_Neg1_PLAYER;

	if ( ! ( object == OBJ_WATER || object == OBJ_OIL ) )
		++m_holding;
}

/*
		Routine to drop an object
*/
void CGameState::drop(short object, short where)
{
	if ( object < 1 || object > LastObjectIndexed() )
	{
		printf("drop(%d, %d) object < 1 || object > LastObjectIndexed()\n", object, where);
		bug(BUG_GameState_cpp+131);  // invalid object
		return;
	}

	if (m_objectLoc1[object] == LOC_Neg1_PLAYER && where == LOC_Neg1_PLAYER )
		return;		// do nothing, must be a juggler or a tela-graber.
									
	if (m_objectLoc1[object] == LOC_Neg1_PLAYER)
		if ( ! ( object == OBJ_WATER || object == OBJ_OIL ) )
			--m_holding;

	if ( m_holding < 0 )
	{
		printf("drop(%d, %d) (m_holding(%d) < 0)\n", object, where, m_holding);
		bug(BUG_GameState_cpp+132);  // invalid condition
	}

	m_objectLoc1[object] = where;
}


/*
		Routine to check for presence
		of dwarves..
*/
short CGameState::dwarfCheck()
{
	short	i;
										
	for (i =1; i < DWARFLAST; ++i)
		if (m_dwarfLoc1[i] == m_location)
			return(i);
	return(0);
}

/*
		Determine liquid in the bottle
		Returns OBJ_WATER
				OBJ_NOTHING
				OBJ_OIL
*/
short CGameState::liq()
{
	short	i, j;
	i=m_objectProperty[OBJ_BOTTLE];		// 0 = water, 1 = empty, 2 oil
										// or
	j=-1-i;								// or
	return(liq2(i>j ? i : j));
}

/*
		Determine liquid at a location
		Returns OBJ_WATER
				OBJ_NOTHING
				OBJ_OIL
*/
short CGameState::liqloc(short location)
{
	if ( ! ValidCaveLocation( location ) )
		return liq2(1);
	if (m_locCondition[location]&LIQUID)
		return(liq2(m_locCondition[location]&OIL));
	else
		return(liq2(1));	// OBJ_NOTHING
}

/*
		Convert	0 to OBJ_WATER		21
				1 to OBJ_NOTHING	 0
				2 to OBJ_OIL		22
*/
short CGameState::liq2(short pbottle)
{
#if 1
	return((1-pbottle)*OBJ_WATER+(pbottle/2)*(OBJ_WATER+OBJ_OIL)); // very clever
#else
	// same as
	switch ( pbottle )
	{
		case 0:	return OBJ_WATER;
		case 1:	return OBJ_NOTHING;
		case 2:	return OBJ_OIL;
	}
#endif
}

/*
		DWARF stuff.
*/
void CGameState::DoDwarves()
{
	short	i, j, kMsg, tri, attack, stick, dwarfsTotal;

	/*
			see if DoDwarves allowed here
	*/
	if (m_newloc1 == LOC_000_Limbo
		|| forced(m_newloc1)
		|| m_locCondition[m_newloc1]&NOPIRAT
		)
		return;
	/*
			see if DoDwarves are active.
	*/
	if (!m_dwarfFlag)
	{
		if (m_newloc1 > LOC_015_hall_mists)
			++m_dwarfFlag;
		return;
	}
	/*
			if first close encounter (of 3rd kind)
			kill 0, 1 or 2
	*/
	if (m_dwarfFlag == 1) 
	{
		if (m_newloc1 < LOC_015_hall_mists || pct (86, 1)) // aw was 95
			return;

		++m_dwarfFlag;

		for (i = 1; i < 3; ++i)
			if (pct (50, 2))
				m_dwarfLoc1[Rrand(1,5)] = LOC_000_Limbo;

		for (i = 1; i < DWARFLAST; ++i) 
		{
			if (m_dwarfLoc1[i] == m_newloc1)
				m_dwarfLoc1[i] = m_dwarfAltLoc;
			m_dwarfOldLoc[i] = m_dwarfLoc1[i];
		}
		SpeakInfoMsg(MSG4_003_dwarf_throws_axe_at_you);
		drop(OBJ_AXE, m_newloc1);
		return;
	}
	dwarfsTotal = attack = stick = 0;
	for (i = 1; i < DWARFMAX; ++i) 
	{
		if (m_dwarfLoc1[i] == LOC_000_Limbo)
			continue;
		/*
				move a DWARF at random.  we don't
				have a matrix around to do it
				as in the original version...
		*/
		for (tri = 1; tri < 20; ++tri) 
		{
			j = Rrand(LOC_015_hall_mists, LOC_121_secret_canyon); /* allowed area */

			if (   j != m_dwarfOldLoc[i]
				&& j != m_dwarfLoc1[i]
				&& ! (i == (DWARFLAST) && m_locCondition[j]&NOPIRAT == 1)
				)
				break;
		}
		if (j == 0)
			j = m_dwarfOldLoc[i];

		m_dwarfOldLoc[i] = m_dwarfLoc1[i];
		m_dwarfLoc1[i] = j;

		if ( (m_dwarfSeen[i] && m_newloc1 >= LOC_015_hall_mists)
			|| m_dwarfLoc1[i] == m_newloc1
			|| m_dwarfOldLoc[i] == m_newloc1
			)
			m_dwarfSeen[i] = 1;
		else
			m_dwarfSeen[i] = 0;

#if _DEBUG && 0
		for ( short nDwNum = 0; nDwNum <= DWARFLAST; ++nDwNum )
		{
			printf("Loc(%d) dw(%d) Loc(%d) seen(%d)\n", m_newloc1, nDwNum, m_dwarfLoc1[nDwNum], m_dwarfSeen[nDwNum]);
		}
		puts("\n");
#endif		

		if (!m_dwarfSeen[i])
			continue;

		m_dwarfLoc1[i] = m_newloc1;

		if (i == DWARFLAST)
		{
			DoPirate();							// Do Pirate stuff
		}
		else if ( Get_DwarfsEnabled() )
		{
			++dwarfsTotal;
			if (m_dwarfOldLoc[i] == m_dwarfLoc1[i]) 
			{
				short nPct = Get_Friendly() ? 88 : 95;	// added by aw --

				++attack;
				if (m_knifeLoc >= 0)
					m_knifeLoc = m_newloc1;
				if ( Rrand(0,999) < nPct * (m_dwarfFlag-2) )
					++stick;
			}
		}
		else
		{
			if ( m_dwarfFlag > 2 )
				m_dwarfFlag = 2;
			if ( i != DWARFLAST )
			{
				m_dwarfLoc1[i] = LOC_000_Limbo;
			}
		}
	}
	if (dwarfsTotal == 0)
		return;
	if (dwarfsTotal > 1)
		printf("There are %d threatening little dwarves in the room with you!\n", dwarfsTotal);
	else
		SpeakInfoMsg(MSG4_004_threatening_dwarf_in_room_you);
	if (attack == 0)
		return;
	if (m_dwarfFlag == 2)
		++m_dwarfFlag;
	if (attack > 1) 
	{
		printf("%d of them throw knives at you!!\n", attack);
		kMsg = MSG4_006_None_hit_you;
	}
	else 
	{
		SpeakInfoMsg(MSG4_005_One_sharp_nasty_knife_thrown_you);
		kMsg = MSG4_052_It_misses;
	}
	if (stick <= 1) 
	{
		SpeakInfoMsg( kMsg+stick );
		if (stick == 0)
			return;
	}
	else
	{
		printf("%d of them get you !!!\n", stick);
	}
	m_oldloc2 = m_newloc1;
	death();
}


/*
		pirate stuff
*/

bool CGameState::PirateCanStealIt( short nObject, short nLoc )
{
	bool bRc = true;
	if (   nObject == OBJ_PYRAMID && nLoc == m_objectLoc1[OBJ_PYRAMID]
		|| nObject == OBJ_EMERALD && nLoc == m_objectLoc1[OBJ_EMERALD]
	   )
	   return false;

	if ( Get_Friendly() )	// added by aw --
	{
		if ( nObject == OBJ_TRIDENT && m_objectLoc1[OBJ_PEARL] == LOC_000_Limbo )
		   return false;

		if ( nObject == OBJ_EGGS && m_objectLoc1[OBJ_TROLL] != LOC_000_Limbo )
		   return false;
	}

	return bRc;
}

void CGameState::DoPirate()
{
	short	j, k;


	if ( ! Get_PirateEnabled() )
			return;

	/*
			see if pirate allowed here
	*/

	if (m_newloc1 == LOC_000_Limbo
		|| forced(m_newloc1)
		|| m_locCondition[m_newloc1]&NOPIRAT
		)
		return;

	if (   m_newloc1 == m_chestLoc1
		|| m_objectProperty[OBJ_CHEST] >= 0
		)
		return;

	// added by aw --
	if ( Get_Friendly() && Get_objectLoc1(OBJ_CHEST) != LOC_000_Limbo )
	{
		if ( pct(95, 3) )
		{
			if ( pct(20, 4) )
				SpeakInfoMsg(MSG4_127_rustling_noises_behind_you);
			return;
		}
	}

	k = 0;
	for (j = FirstTreasure(); j <= LastTreasure(); ++j)
	{
		if ( PirateCanStealIt( j, m_newloc1 ) )
		{
			if (toting(j))
				goto stealit;
			if (here(j))
				++k;
		}
	}
	if (m_tally1 == m_tally2+1 && k == 0 && m_objectLoc1[OBJ_CHEST] == LOC_000_Limbo &&
		here(OBJ_LAMP) && m_objectProperty[OBJ_LAMP] == 1) 
	{
		SpeakInfoMsg(MSG4_186_rustling_noises_from_darkness);
		move(OBJ_CHEST, m_chestLoc1, LOC_000_MOVEABLE);
		move(OBJ_MESSAGE, m_chestLoc2, LOC_Neg1_FIXED);
		m_dwarfLoc1[DWARFLAST] = m_chestLoc1;
		m_dwarfOldLoc[DWARFLAST] = m_chestLoc1;
		m_dwarfSeen[DWARFLAST] = 0;
		return;
	}
	if (m_dwarfOldLoc[DWARFLAST] != m_dwarfLoc1[DWARFLAST] && pct(20, 1)) // aw, was 20
	{
		SpeakInfoMsg(MSG4_127_rustling_noises_behind_you);
		return;
	}
	return;
										
	stealit:
										
	SpeakInfoMsg(MSG4_128_from_shadows_pirate_takes_all_booty);
	if (m_objectLoc1[OBJ_MESSAGE] == LOC_000_Limbo)
		move(OBJ_CHEST, m_chestLoc1, LOC_000_MOVEABLE);

	move(OBJ_MESSAGE, m_chestLoc2, LOC_Neg1_FIXED);

	for (j = FirstTreasure(); j <= LastTreasure(); ++j) 
	{
		if ( ! PirateCanStealIt( j, m_newloc1 ) )
			continue;

		if (at(j) && m_objectLoc2[j] == LOC_000_MOVEABLE) // not fixed
			carry(j, m_newloc1);
		if (toting(j))
			drop(j, m_chestLoc1);
	}
	m_dwarfLoc1[DWARFLAST] = m_chestLoc1;
	m_dwarfOldLoc[DWARFLAST] = m_chestLoc1;
	m_dwarfSeen[DWARFLAST] = 0;
}


//	From: TrVerb.cpp



short	actmsg[] = 
{		/* action messages	*/
	/* VRB_NULL   0 */	MSG4_000_No_Message,
	/* VRB_TAKE   1 */	MSG4_024_already_carrying_it,
	/* VRB_DROP   2 */	MSG4_029_not_carrying_it,
	/* VRB_SAY    3 */	MSG4_061_What,
	/* VRB_OPEN   4 */	MSG4_033_don_t_know_how_lock_or_unlock_such_thing,
	/* VRB_NOTHING 5 */	MSG4_042_Nothing_happens,
	/* VRB_LOCK   6 */	MSG4_033_don_t_know_how_lock_or_unlock_such_thing,
	/* VRB_ON     7 */	MSG4_038_no_source_light,
	/* VRB_OFF    8 */	MSG4_038_no_source_light,
	/* VRB_WAVE   9 */	MSG4_042_Nothing_happens,
	/* VRB_CALM  10 */	MSG4_014_Would_you_care_explain_how,
	/* VRB_WALK  11 */	MSG4_043_Where,
	/* VRB_KILL  12 */	MSG4_110_don_t_be_ridiculous,
	/* VRB_POUR  13 */	MSG4_029_not_carrying_it,
	/* VRB_EAT   14 */	MSG4_110_don_t_be_ridiculous,
	/* VRB_DRINK 15 */	MSG4_073_You_drink_from_stream_water,
	/* VRB_RUB   16 */	MSG4_075_Rubbing_electric_lamp,
	/* VRB_THROW 17 */	MSG4_029_not_carrying_it,
	/* VRB_QUIT  18 */	MSG4_013_don_t_understand_that,
	/* VRB_FIND  19 */	MSG4_059_only_tell_you_what_you_see,
	/* VRB_INV   20 */	MSG4_059_only_tell_you_what_you_see,
	/* VRB_FEED  21 */	MSG4_174_nothing_here_eat,
	/* VRB_FILL  22 */	MSG4_109_can_t_fill_that,
	/* VRB_BLAST 23 */	MSG4_067_Blasting_requires_dynamite,
	/* VRB_SCORE 24 */	MSG4_013_don_t_understand_that,
	/* VRB_FRIENDLY 25 */	MSG4_147_I_don_t_know_how,
	/* VRB_BRIEF 26 */	MSG4_155_On_what,
	/* VRB_READ  27 */	MSG4_195_don_t_understand,
	/* VRB_BREAK 28 */	MSG4_146_beyond_power_do,
	/* VRB_WAKE  29 */	MSG4_110_don_t_be_ridiculous,
	/* VRB_SUSPEND 30 */	MSG4_013_don_t_understand_that,
	/* VRB_HOURS 31 */	MSG4_228_CaveOpen_24_7_ForNow,			// end of orignal table
	/* VRB_LOG   32 */	MSG4_155_On_what,
	/* VRB_LOOK  33 */	MSG4_059_only_tell_you_what_you_see,
	/* RESERVED  34 */	MSG4_000_No_Message,
	/* RESERVED  35  */	MSG4_000_No_Message,
	/* RESERVED  36 */	MSG4_000_No_Message,
	/* VRB_WATER 37 */	MSG4_074_bottle_now_empty,
	/* VRB_OIL   38 */	MSG4_074_bottle_now_empty,
	/* VRB_MAP   39 */	MSG4_000_No_Message,
	/* VRB_FEE   40 */	MSG4_042_Nothing_happens,
	/* VRB_FIE   41 */	MSG4_042_Nothing_happens,
	/* VRB_FOE   42 */	MSG4_042_Nothing_happens,
	/* VRB_FOO   43 */	MSG4_042_Nothing_happens,
	/* VRB_FUM   44 */	MSG4_000_No_Message,
	/* RESERVED  45 */	MSG4_000_No_Message,
	/* RESERVED  46 */	MSG4_000_No_Message,
	/* RESERVED  47 */	MSG4_000_No_Message,
	/* RESERVED  48 */	MSG4_000_No_Message,
	/* RESERVED  49 */	MSG4_000_No_Message,
	/* VRB_SAVE  50 */	MSG4_000_No_Message,
	/* VRB_RESTORE 51 */	MSG4_000_No_Message,
	/* VRB_RESET 52 */	MSG4_000_No_Message,
	/* VRB_CRYPT 53 */	MSG4_000_No_Message,
	/* VRB_INPUT 54 */	MSG4_000_No_Message,
	/* VRB_VOCABLENGTH 55 */	MSG4_000_No_Message,
	/* VRB_HINTS 56 */	MSG4_000_No_Message,
	/* VRB_AUTOCP 57 */	MSG4_000_No_Message,
	/* RESERVED  58 */	MSG4_000_No_Message,
};

#define NUM_ACTMSG (sizeof(actmsg) / sizeof(actmsg[0]))

/*
		Routine to process a transitive verb
*/
void CGameState::trverb()
{
	switch(e_verb)
	{
		case VRB_FEE:
		case VRB_FIE:
		case VRB_FOE:
		case VRB_FOO:
		case VRB_FUM:

		case VRB_AUTOCP:
		case VRB_SUSPEND:
		case VRB_CHECKPOINT:
		case VRB_RESTORE:
		case VRB_INPUT:

		case VRB_CRYPT:
			itverb();
			break;

		case VRB_CALM:
		case VRB_WALK:
		case VRB_QUIT:
		case VRB_SCORE:
		case VRB_BRIEF:
		case VRB_HOURS:
		case VRB_RESET:
		case VRB_LOG:
		case VRB_FRIENDLY:
		case VRB_HINTS:
		case VRB_VOCABLENGTH:
			actspk(e_verb);
			break;

		case VRB_TAKE:
			vtake();
			break;
		case VRB_DROP:
			vdrop();
			break;
		case VRB_OPEN:
		case VRB_LOCK:
			vopen();
			break;
		case VRB_SAY:
			vsay();
			break;
		case VRB_NOTHING:
			SpeakInfoMsg(MSG4_054_OK);
			break;
		case VRB_ON:
			von();
			break;
		case VRB_OFF:
			voff();
			break;
		case VRB_WAVE:
			vwave();
			break;
		case VRB_KILL:
			vkill();
			break;
		case VRB_WATER:
			e_verb	= VRB_POUR;
			e_object = OBJ_WATER;
			vpour();
			break;
		case VRB_OIL:
			e_verb	= VRB_POUR;
			e_object = OBJ_OIL;
		case VRB_POUR:
			vpour();
			break;
		case VRB_EAT:
			veat();
			break;
		case VRB_DRINK:
			vdrink();
			break;
		case VRB_RUB:
			if (e_object != OBJ_LAMP)
				SpeakInfoMsg(MSG4_076_Nothing_happens);
			else
				actspk(VRB_RUB);
			break;
		case VRB_THROW:
			vthrow();
			break;
		case VRB_FEED:
			vfeed();
			break;
		case VRB_FIND:
		case VRB_INVENTORY:
			vfind();
			break;
		case VRB_FILL:
			vfill();
			break;
		case VRB_READ:
			vread();
			break;
		case VRB_BLAST:
			vblast();
			break;
		case VRB_BREAK:
			vbreak();
			break;
		case VRB_WAKE:
			vwake();
			break;
		case VRB_MAP:
			cavemap();
			break;
		default:
			printf("This verb is not implemented yet.\n");
	}
}

/*
		Routine to process a transitive verb
*/
/*
		CARRY TAKE etc.
*/
void CGameState::vtake()
{
	short msg = MSG4_025_You_can_t_be_serious;
										
	if (toting(e_object)) 
	{
		actspk(e_verb);
		return;
	}
	/*
			special case objects and m_objectLoc2 objects
	*/
	if (e_object == OBJ_PLANT && m_objectProperty[OBJ_PLANT] <= 0)
		msg = MSG4_115_plant_deep_roots_cannot_be_pulled_free;
	if (e_object == OBJ_BEAR && m_objectProperty[OBJ_BEAR] == 1)
		msg = MSG4_169_bear_still_chained_wall;
	if (e_object == OBJ_CHAIN && m_objectProperty[OBJ_BEAR] != 0)
		msg = MSG4_170_chain_still_locked;
	if (m_objectLoc2[e_object]) 
	{
		SpeakInfoMsg(msg);
		return;
	}
	/*
			special case for liquids
	*/
	if (e_object == OBJ_WATER || e_object == OBJ_OIL) 
	{
		if (!here(OBJ_BOTTLE) || liq() != e_object) 
		{
			e_object = OBJ_BOTTLE;
			if (toting(OBJ_BOTTLE)&&m_objectProperty[OBJ_BOTTLE] == 1)	// empty?
			{
				vfill();
				return;
			}
			if (m_objectProperty[OBJ_BOTTLE] != 1)			//	Not empty?
				msg = MSG4_105_bottle_already_full;
			if (!toting(OBJ_BOTTLE))
				msg = MSG4_104_have_nothing_carry_it;
			SpeakInfoMsg(msg);
			return;
		}
		e_object = OBJ_BOTTLE;
	}
	if (m_holding > m_holdMax)
	{
		SpeakInfoMsg(MSG4_092_can_t_carry_anything_more_have_drop_something_first);
		return;
	}
	/*
			special case for OBJ_BIRD.
	*/
	if (e_object == OBJ_BIRD && m_objectProperty[OBJ_BIRD] == 0) 
	{
		if (toting(OBJ_ROD)) 
		{
			SpeakInfoMsg(MSG4_026_bird_was_unafraid_when_you_entered);
			return;
		}
		if (!toting(OBJ_CAGE)) 
		{
			SpeakInfoMsg(MSG4_027_can_catch_bird_but_not_carry_it);
			return;
		}
		m_objectProperty[OBJ_BIRD] = 1;
		m_hintAvail &= ~HINTB;
	}
	if ( (e_object == OBJ_BIRD
		|| e_object == OBJ_CAGE) &&
		m_objectProperty[OBJ_BIRD] != 0)
	{
		carry((OBJ_BIRD+OBJ_CAGE)-e_object,m_location);
		carry(e_object,m_location);
		SpeakInfoMsg(MSG4_054_OK);
		return;
	}
	/*
			handle liquid in OBJ_BOTTLE
	*/
	if (e_object == OBJ_BOTTLE && liq() != 0)
	{
		carry((OBJ_BOTTLE+liq())-e_object,m_location);
		carry(e_object,m_location);
//		m_objectLoc1[liq()] = LOC_Neg1_PLAYER;
		SpeakInfoMsg(MSG4_054_OK);
		return;
	}
	carry(e_object,m_location);
	SpeakInfoMsg(MSG4_054_OK);
}

/*
		DROP etc.
*/
void CGameState::vdrop()
{
	short msg = MSG4_054_OK;
										
	/*
			check for dynamite
	*/
	if (toting(OBJ_ROD2) && e_object == OBJ_ROD && !toting(OBJ_ROD))
		e_object = OBJ_ROD2;
	if (!toting(e_object))
	{
		actspk(e_verb);
		return;
	}
	/*
			SNAKE and BIRD
	*/
	if (e_object == OBJ_BIRD && here(OBJ_SNAKE)) 
	{
		SpeakInfoMsg(MSG4_030_bird_attacks_green_snake_drives_away);
		msg = MSG4_000_No_Message;
		if (m_closed)
			dwarfend();
		dstroy(OBJ_SNAKE);
		m_objectProperty[OBJ_SNAKE] = -1;
		m_hintAvail &= ~HINTS;
	}
	/*
			COINS and vending machine
	*/
	else if (e_object == OBJ_COINS && here(OBJ_VENDING)) 
	{
		dstroy(OBJ_COINS);
		drop(OBJ_BATTERIES,m_location);
		SpeakObjDesc(OBJ_BATTERIES,0);
		return;
	}
	/*
			OBJ_BIRD and OBJ_DRAGON (ouch!!)
	*/
	else if (e_object == OBJ_BIRD && at(OBJ_DRAGON) && m_objectProperty[OBJ_DRAGON] == 0)
	{
		SpeakInfoMsg(MSG4_154_bird_attacks_dragon_gets_burnt);
		msg = MSG4_000_No_Message;
		dstroy(OBJ_BIRD);
		m_objectProperty[OBJ_BIRD] = 0;
		if (m_objectLoc1[OBJ_SNAKE] != LOC_000_Limbo)
			++m_tally2;
		return;
	}
	/*
			OBJ_BEAR and OBJ_TROLL
	*/
	if (e_object == OBJ_BEAR && at(OBJ_TROLL)) 
	{
		SpeakInfoMsg(MSG4_163_bear_lumbers_toward_troll_who_shriek);
		move(OBJ_TROLL,LOC_000_Limbo, LOC_000_MOVEABLE);
		move(OBJ_TROLL2,LOC_117_sw_chasm, LOC_122_ne_chasm);
		juggle(OBJ_CHASM);
		m_objectProperty[OBJ_TROLL] = 2;
	}
	/*
			OBJ_VASE
	*/
	else if (e_object == OBJ_VASE) 
	{
		if (m_location == LOC_096_soft_room)
			SpeakInfoMsg(MSG4_054_OK);
		else 
		{
			m_objectProperty[OBJ_VASE] = at(OBJ_PILLOW) ? 0 : 2;
			SpeakObjDesc(OBJ_VASE,m_objectProperty[OBJ_VASE]+1);
			if (m_objectProperty[OBJ_VASE] != 0)
				m_objectLoc2[OBJ_VASE] = LOC_Neg1_FIXED;
		}
		msg = MSG4_000_No_Message;
	}
	/*
			handle liquid and OBJ_BOTTLE
	*/
	if (e_object == liq())
		e_object = OBJ_BOTTLE;
	if (e_object == OBJ_BOTTLE && liq() != OBJ_NOTHING)
		m_objectLoc1[liq()] = LOC_000_Limbo;
	/*
			handle OBJ_BIRD and OBJ_CAGE
	*/
	if (e_object == OBJ_CAGE && m_objectProperty[OBJ_BIRD] != 0)
		drop(OBJ_BIRD,m_location);
	if (e_object == OBJ_BIRD)
		m_objectProperty[OBJ_BIRD] = 0;

	drop(e_object,m_location);

	if ( msg )
		SpeakInfoMsg (msg);
}

/*
		LOCK, UNLOCK, OPEN, CLOSE etc.
*/

void CGameState::vopen()
{
	short msg = MSG4_000_No_Message;
	short oyclam;
										
	switch(e_object) 
	{
		case OBJ_MAP:
			if ( ! m_bMapOpen )
			{
				SpeakInfoMsg(MSG4_217_Map_out_of_date);
				m_bMapOpen = yes(MSG4_216_Cost_15_points_open_map, MSG4_200_Is_this_acceptable, MSG4_218_Proably_good_choice);
			}
			if ( m_bMapOpen )
			{
				++m_hintTaken;
				DisplayThisLocationMap( m_location );
			}
			break;
		case OBJ_CLAM:
		case OBJ_OYSTER:
			oyclam = (e_object == OBJ_OYSTER ? 1 : 0);
			if (e_verb == VRB_LOCK)
				msg = MSG4_061_What;
			else if (!toting(OBJ_TRIDENT))
					msg = MSG4_122_don_t_have_anything_to_open_clam+oyclam;
			else if (toting(e_object))
					msg = MSG4_120_advise_put_down_clam_before_opening_it_STRAIN+oyclam;
			else 
			{
				msg = MSG4_124_pearl_falls_out_of_clam+oyclam;
				dstroy(OBJ_CLAM);
				drop(OBJ_OYSTER,m_location);
				drop(OBJ_PEARL,LOC_105_cul_de_sac);
			}
			break;
		case OBJ_DOOR:
			msg = (m_objectProperty[OBJ_DOOR] == 1 ? MSG4_054_OK : MSG4_111_rusty_door_wont_open);
			break;
		case OBJ_CAGE:
			msg = MSG4_032_no_lock;
			break;
		case OBJ_KEYS:
			msg = MSG4_055_can_t_unlock_keys;
			break;
		case OBJ_CHAIN:
			if (!here(OBJ_KEYS))
				msg = MSG4_031_no_keys;
			else if (e_verb == VRB_LOCK) 
			{
				if (m_objectProperty[OBJ_CHAIN] != 0)
					msg = MSG4_034_already_locked;
				else if (m_location != LOC_130_barren_room)
						msg = MSG4_173_nothing_here_chain_can_be_locked;
				else 
				{
					m_objectProperty[OBJ_CHAIN] = 2;
					if (toting(OBJ_CHAIN))
						drop(OBJ_CHAIN,m_location);
					m_objectLoc2[OBJ_CHAIN] = LOC_Neg1_FIXED;
					msg = MSG4_172_chain_now_locked;
				}
			}
			else 
			{
				if (m_objectProperty[OBJ_BEAR] == 0) // aw 400 ????
					msg = MSG4_041_no_way_past_bear_unlock_chain;
				else if (m_objectProperty[OBJ_CHAIN] == 0)
						msg = MSG4_037_already_unlocked;
				else 
				{
					m_objectProperty[OBJ_CHAIN] = 0;
					m_objectLoc2[OBJ_CHAIN] = LOC_000_Limbo;
					if (m_objectProperty[OBJ_BEAR] != 3)
						m_objectProperty[OBJ_BEAR] = 2;
					m_objectLoc2[OBJ_BEAR] = 2-m_objectProperty[OBJ_BEAR]; 	// ?? when bear prob is 2,free the bear
					msg = MSG4_171_chain_now_unlocked;
				}
			}
			break;
		case OBJ_GRATE:
			if (!here(OBJ_KEYS))
				msg = MSG4_031_no_keys;
			else if (m_closing) 
			{
				if (!m_panic) 
				{
					m_clock2 = 15;
					++m_panic;
				}
				msg = MSG4_130_exit_closed_leave_via_main_office;
			}
			else 
			{
				msg = MSG4_034_already_locked+m_objectProperty[OBJ_GRATE];
				m_objectProperty[OBJ_GRATE] = (e_verb == VRB_LOCK ? 0 : 1);
				msg += 2*m_objectProperty[OBJ_GRATE];
			}
			break;
		default:
			msg = MSG4_033_don_t_know_how_lock_or_unlock_such_thing;
	}
	if ( msg )
		SpeakInfoMsg(msg);
}

/*
		SAY etc.
*/
void CGameState::vsay()
{
	short wtype,wval;
										
	analyze(e_word1,&wtype,&wval, false);
	printf("Okay.\n%s\n",wval == VRB_SAY ? e_word2 : e_word1);
}

/*
		ON etc.
*/
void CGameState::von()
{
	if ( ! ValidCaveLocation( m_location ) )
	{
		printf("von bad m_location(%d)\n", m_location);
		bug(BUG_TrVerb_Cpp+21);
	}
	if (!here(OBJ_LAMP))
		actspk(e_verb);
	else if (m_limit<0)
			SpeakInfoMsg(MSG4_184_lamp_has_run_out_power);
	else 
	{
		m_objectProperty[OBJ_LAMP] = 1;
		SpeakInfoMsg(MSG4_039_lamp_now_on);
		if (m_wasDark) 
		{
			m_wasDark = 0;
			describe(describeTypeLong);
			++m_locVisitedCnt[m_location];
			descitem();
		}
	}
}

/*
		OFF etc.
*/
void CGameState::voff()
{
	if (!here(OBJ_LAMP))
		actspk(e_verb);
	else 
	{
		m_objectProperty[OBJ_LAMP] = 0;
		SpeakInfoMsg(MSG4_040_lamp_now_off);
	}
}

/*
		WAVE etc.
*/
void CGameState::vwave()
{
	if (!toting(e_object) &&
		(e_object != OBJ_ROD || !toting(OBJ_ROD2)))
		SpeakInfoMsg(MSG4_029_not_carrying_it);
	else if (e_object != OBJ_ROD || !at(OBJ_FISSURE) ||
			!toting(e_object) || m_closing)
			actspk(e_verb);
	else 
	{
		m_objectProperty[OBJ_FISSURE] = 1-m_objectProperty[OBJ_FISSURE];
		SpeakObjDesc(OBJ_FISSURE,2-m_objectProperty[OBJ_FISSURE]);
	}
}

/*
		ATTACK, KILL etc.
*/
void CGameState::vkill()
{
	short msg = MSG4_000_No_Message;
	short i;
										
	switch(e_object) 
	{
		case OBJ_BIRD:
			if (m_closed)
				msg = MSG4_137_leave_poor_bird_alone;
			else 
			{
				dstroy(OBJ_BIRD);
				m_objectProperty[OBJ_BIRD] = 0;
				if (m_objectLoc1[OBJ_SNAKE] == LOC_019_hall_mt_king)
					++m_tally2;
				msg = MSG4_045_bird_now_dead_Its_body_disappears;
			}
			break;
		case 0:
			msg = MSG4_044_There_nothing_here_attack;
			break;
		case OBJ_CLAM:
		case OBJ_OYSTER:
			msg = MSG4_150_shell_very_strong_impervious;
			break;
		case OBJ_SNAKE:
			msg = MSG4_046_Attacking_snake_doesn_t_work_very_dangerous;
			break;
		case OBJ_DWARF:
			if (m_closed)
				dwarfend();
			msg = MSG4_049_what_with_bare_hands;
			break;
		case OBJ_TROLL:
			msg = MSG4_157_Trolls_skin_as_tough_as_rhinoceros;
			break;
		case OBJ_BEAR:
			msg = MSG4_165_what_your_bare_hands_Agains_HIS_bear_hands+(m_objectProperty[OBJ_BEAR]+1)/2;
			break;
		case OBJ_DRAGON:
			if (m_objectProperty[OBJ_DRAGON] != 0)
			{
				msg = MSG4_167_For_crying_out_loud_poor_thing_already_dead;
				break;
			}
			if (!yes(MSG4_049_what_with_bare_hands,0,0))
				break;
			SpeakObjDesc(OBJ_DRAGON,1);
			m_objectProperty[OBJ_DRAGON] = 2;
			m_objectProperty[OBJ_RUG] = 0;
			move(OBJ_DRAGON,LOC_120_secret_canyon, LOC_Neg1_FIXED);
			move(OBJ_RUG,LOC_120_secret_canyon, LOC_000_MOVEABLE);
			for( i = 1; i <= LastObjectIndexed(); ++i )
				if (   m_objectLoc1[i] == LOC_119_secret_canyon
					|| m_objectLoc1[i] == LOC_121_secret_canyon)
					move(i,LOC_120_secret_canyon, LOC_Neg2_NO_CHANGE);
			m_newloc1 = LOC_120_secret_canyon;
			return;
		default:
			actspk(e_verb);
			return;
	}
	if ( msg )
		SpeakInfoMsg(msg);
}

/*
		POUR
*/
void CGameState::vpour()
{
	if (	e_object == OBJ_BOTTLE
		||	e_object == OBJ_NONE
		)
		e_object = liq();
	if (e_object == OBJ_NONE) 
	{
		needobj();
		return;
	}
	if (!toting(e_object)) 
	{
		actspk(e_verb);
		return;
	}
	if (e_object != OBJ_OIL && e_object != OBJ_WATER) 
	{
		SpeakInfoMsg(MSG4_078_You_can_t_pour_that);
		return;
	}
	m_objectProperty[OBJ_BOTTLE] = 1;		// bottle_prop_empty
	m_objectLoc1[e_object] = LOC_000_Limbo;
	if (at(OBJ_PLANT)) 
	{
		if (e_object != OBJ_WATER)
			SpeakInfoMsg(MSG4_112_plant_shakes_oil_off_wants_Water);
		else 
		{
			SpeakObjDesc(OBJ_PLANT,m_objectProperty[OBJ_PLANT]+1);
			m_objectProperty[OBJ_PLANT] = (m_objectProperty[OBJ_PLANT]+2)%6;
			m_objectProperty[OBJ_PLANT2] = m_objectProperty[OBJ_PLANT]/2;
			describe();
			// descitem(); //aw ???
		}
	}
	else if (at(OBJ_DOOR)) 
	{
		m_objectProperty[OBJ_DOOR] = (e_object == OBJ_OIL ? 1 : 0);
		SpeakInfoMsg(MSG4_113_hinges_are_rusted_now_won_t_budge+m_objectProperty[OBJ_DOOR]);
	}
	else
		SpeakInfoMsg(MSG4_077_bottle_empty_ground_wet);
}

/*
		EAT
*/
void CGameState::veat()
{
	short msg;
										
	switch(e_object) 
	{
		case OBJ_FOOD:
			dstroy(OBJ_FOOD);
			msg = MSG4_072_Thank_you_delicious;
			break;
		case OBJ_BIRD: case OBJ_SNAKE: case OBJ_CLAM: case OBJ_OYSTER:
		case OBJ_DWARF: case OBJ_DRAGON: case OBJ_TROLL: case OBJ_BEAR:
			msg = MSG4_071_lost_my_appetite;
			break;
		default:
			actspk(e_verb);
			return;
	}
	SpeakInfoMsg(msg);
}

/*
		DRINK
*/
void CGameState::vdrink()
{
	if (e_object != OBJ_WATER)
		SpeakInfoMsg(MSG4_110_don_t_be_ridiculous);
	else if (liq() != OBJ_WATER || !here(OBJ_BOTTLE))
			actspk(e_verb);
	else 
	{
		m_objectProperty[OBJ_BOTTLE] = 1;		// prop_empty
		m_objectLoc1[OBJ_WATER] = LOC_000_Limbo;
		SpeakInfoMsg(MSG4_074_bottle_now_empty);
	}
}

/*
		THROW etc.
*/
void CGameState::vthrow()
{
	short msg = MSG4_000_No_Message;
	short i;
										
	if (toting(OBJ_ROD2) && e_object == OBJ_ROD && !toting(OBJ_ROD))
		e_object = OBJ_ROD2;

	if (!toting(e_object)) 
	{
		actspk(e_verb);
		return;
	}
	/*
			treasure to TROLL
	*/
	if (at(OBJ_TROLL) && e_object >= OBJ_NUGGET && e_object<=LastObjectIndexed()) 
	{
		SpeakInfoMsg(MSG4_159_troll_catches_treasure_scurries_away);
		drop(e_object,LOC_000_Limbo);
		move(OBJ_TROLL,LOC_000_Limbo, LOC_000_MOVEABLE);
		move(OBJ_TROLL2,LOC_117_sw_chasm, LOC_122_ne_chasm);
		juggle(OBJ_CHASM);
		return;
	}
	/*
			feed the bears...
	*/
	if (e_object == OBJ_FOOD && here(OBJ_BEAR)) 
	{
		e_object = OBJ_BEAR;
		vfeed();
		return;
	}
	/*
			if not OBJ_AXE, same as drop...
	*/
	if (e_object != OBJ_AXE) 
	{
		vdrop();
		return;
	}


	/*
			********************   AXE is THROWN ***********************
	*/

	e_word1[0] = '\0';	// clear axe because we are going to replace verb
	e_word2[0] = '\0';	// clear axe because we are going to replace object

	/*
			at a OBJ_DWARF...
	*/
	if (i = dwarfCheck()) 
	{
		short nPct = Get_Friendly() ? 44 : 33;	// added by aw --
		msg = MSG4_048_You_attack_dwarf_he_dodges_out_way;
		if ( pct(nPct, 11) ) 
		{
			m_dwarfSeen[i] = m_dwarfLoc1[i] = 0;
			msg = MSG4_047_You_killed_dwarf;
			++m_dwarfsKilled;
			if (m_dwarfsKilled == 1)
				msg = MSG4_149_killed_Dwarf_vanished_smoke;
		}
	}
	/*
			at a OBJ_DRAGON...
	*/
	else if (at(OBJ_DRAGON) && m_objectProperty[OBJ_DRAGON] == 0) 
			msg = MSG4_152_axe_bounces_off_dragon_scales;
	/*
			at the OBJ_TROLL...
	*/
	else if (at(OBJ_TROLL))
			msg = MSG4_158_troll_catches_axe_tosses_it_back;
	/*
			at the OBJ_BEAR...
	*/
	else if (here(OBJ_BEAR) && m_objectProperty[OBJ_BEAR] == 0) 
	{
		SpeakInfoMsg(MSG4_164_axe_misses_lands_near_bear_where_you_can_t_get_it);
		drop(OBJ_AXE,m_location);
		m_objectLoc2[OBJ_AXE] = LOC_Neg1_FIXED;
		m_objectProperty[OBJ_AXE] = 1;
		juggle(OBJ_BEAR);
		return;
	}
	/*
			otherwise it is an attack
	*/
	else 
	{
		e_verb = VRB_KILL;
		e_object = OBJ_NONE;
		itverb();
//		return;
	}
	/*
			handle the left over OBJ_AXE...
	*/
	SpeakInfoMsg(msg);
	drop(OBJ_AXE,m_location);
	describe();
//	descitem(); // aw ???
}

/*
		INVENTORY, FIND etc.
*/
void CGameState::vfind()
{
	short msg;
	if (toting(e_object))
		msg = MSG4_024_already_carrying_it;
	else if (m_closed)
			msg = MSG4_138_whatever_you_want_is_here_somewhere;
	else if (dwarfCheck() && m_dwarfFlag >= 2 && e_object == OBJ_DWARF)
			msg = MSG4_094_what_you_want_right_here;
	else if (at(e_object) ||
			(liq() == e_object && here(OBJ_BOTTLE)) ||
			e_object == liqloc(m_location))
			msg = MSG4_094_what_you_want_right_here;
	else 
	{
		actspk(e_verb);
		return;
	}
	SpeakInfoMsg(msg);
}

/*
		FILL
*/
void CGameState::vfill()
{
	short msg;
	short i;
										
	if ( ! ValidCaveLocation( m_location ) )
	{
		printf("vfill() bad m_location(%d)\n", m_location);
		bug(BUG_TrVerb_Cpp+61);
	}

	switch(e_object) 
	{
		case OBJ_BOTTLE:
			if (liq() != 0)
				msg = MSG4_105_bottle_already_full;
			else if (liqloc(m_location) == 0)
					msg = MSG4_106_nothing_here_fill_bottle;
			else 
			{
				m_objectProperty[OBJ_BOTTLE] = m_locCondition[m_location] & OIL;
				i = liq();
				if (toting(OBJ_BOTTLE))
					m_objectLoc1[i] = LOC_Neg1_PLAYER;
				msg = (i == OBJ_OIL ? MSG4_108_bottle_now_full_oil : MSG4_107_bottle_now_full_water);
			}
			break;
		case OBJ_VASE:
			if (liqloc(m_location) == 0) 
			{
				msg = MSG4_144_nothing_here_to_fill_vase;
				break;
			}
			if (!toting(OBJ_VASE)) 
			{
				msg = MSG4_029_not_carrying_it;
				break;
			}
			SpeakInfoMsg(MSG4_145_temperature_change_shattered_vase);
			vdrop();
			return;
		default:
			msg = MSG4_029_not_carrying_it;
	}
	SpeakInfoMsg(msg);
}

/*
		FEED
*/
void CGameState::vfeed()
{
	short msg;
										
	switch(e_object) 
	{
		case OBJ_BIRD:
			msg = MSG4_100_It_s_not_hungry_and_You_have_no_bird_seed;
			break;
		case OBJ_DWARF:
			if (!here(OBJ_FOOD)) 
			{
				actspk(e_verb);
				return;
			}
			++m_dwarfFlag;
			msg = MSG4_103_fool_Dwarves_eat_only_coal;
			break;
		case OBJ_BEAR:
			if (!here(OBJ_FOOD)) 
			{
				if (m_objectProperty[OBJ_BEAR] == 0)
					msg = MSG4_102_nothing_here_it_wants_eat_Except_you;
				else if (m_objectProperty[OBJ_BEAR] == 3)
						msg = MSG4_110_don_t_be_ridiculous;
				else 
				{
					actspk(e_verb);
					return;
				}
				break;
			}
			dstroy(OBJ_FOOD);
			m_objectProperty[OBJ_BEAR] = 1;

			m_objectLoc2[OBJ_AXE] = LOC_000_MOVEABLE;
			m_objectProperty[OBJ_AXE] = 0;

			msg = MSG4_168_bear_wolfs_down_food_becomes_friendly;
			break;
		case OBJ_DRAGON:
			msg = (m_objectProperty[OBJ_DRAGON] != 0 ? MSG4_110_don_t_be_ridiculous : MSG4_102_nothing_here_it_wants_eat_Except_you);
			break;
		case OBJ_TROLL:
			msg = MSG4_182_Gluttony_not_Troll_vice;
			break;
		case OBJ_SNAKE:
			if (m_closed || !here(OBJ_BIRD)) 
			{
				msg = MSG4_102_nothing_here_it_wants_eat_Except_you;
				break;
			}
			msg = MSG4_101_snake_has_now_devoured_bird;
			dstroy(OBJ_BIRD);
			m_objectProperty[OBJ_BIRD] = 0;
			++m_tally2;
			break;
		default:
			msg = MSG4_014_Would_you_care_explain_how;
	}
	SpeakInfoMsg(msg);
}

/*
		READ etc.
*/
void CGameState::vread()
{
	short msg = MSG4_000_No_Message;
										
	if (dark()) 
	{
		printf("I see no %s here.\n",probj(e_object));
		return;
	}
	switch(e_object) 
	{
		case OBJ_MAP:
			msg = MSG4_215_Handle_with_care;
			break;
		case OBJ_MAGAZINE:
			msg = MSG4_190_magazine_in_Dwarvish;
			break;
		case OBJ_TABLET:
			msg = MSG4_196_Congratulations_bringing_light_into_dark_room;
			break;
		case OBJ_MESSAGE:
			msg = MSG4_191_not_maze_where_pirate_treasure_chest;
			break;
		case OBJ_OYSTER:
			if (!toting(OBJ_OYSTER) || !m_closed)
				break;
			yes(MSG4_192_looks_like_clue_cost_you_10_points,MSG4_193_says_something_strange_about_this_place,MSG4_054_OK);
			return;
		default:
			;
	}
	if (msg)
		SpeakInfoMsg(msg);
	else
		actspk(e_verb);
}

/*
		BLAST etc.
*/
void CGameState::vblast()
{
	if (m_objectProperty[OBJ_ROD2] < 0 || !m_closed)
		actspk(e_verb);
	else 
	{
		m_bonus = MSG4_133_loud_explosion_hole_You_can_exit;
		if (m_location == LOC_115_ne_end_repository)
			m_bonus = MSG4_134_loud_explosion_burying_everything;
		if (here(OBJ_ROD2))
			m_bonus = MSG4_135_loud_explosion_you_are_splashed_across_walls;
		SpeakInfoMsg(m_bonus);
		normend();
	}
}

/*
		BREAK etc.
*/
void CGameState::vbreak()
{
	short msg;
	if (e_object == OBJ_MIRROR) 
	{
		msg = MSG4_148_too_far_up_to_reach;
		if (m_closed) 
		{
			SpeakInfoMsg(MSG4_197_You_strike_mirror_shatters);
			dwarfend();
		}
	}
	else if (e_object == OBJ_VASE && m_objectProperty[OBJ_VASE] == 0) 
	{
		msg = MSG4_198_vase_hurled_ground;
		if (toting(OBJ_VASE))
			drop(OBJ_VASE,m_location);
		m_objectProperty[OBJ_VASE] = 2;
		m_objectLoc2[OBJ_VASE] = LOC_Neg1_FIXED;
	}
	else 
	{
		actspk(e_verb);
		return;
	}
	SpeakInfoMsg(msg);
}

/*
		WAKE etc.
*/
void CGameState::vwake()
{
	if (e_object != OBJ_DWARF || !m_closed)
		actspk(e_verb);
	else 
	{
		SpeakInfoMsg(MSG4_199_woke_nearest_Dwarf);
		dwarfend();
	}
}

/*
		Routine to speak default verb message
*/
void CGameState::actspk(short pverb)
{
	short msg;
										
	if (pverb < 1 || pverb >= NUM_ACTMSG)
	{
		printf("actspk() bad m_location(%d)\n", m_location );
		bug(BUG_TrVerb_Cpp+81);
		return;
	}
	msg = actmsg[pverb];
	if (msg)
		SpeakInfoMsg(msg);
}

/*
		Routine to indicate no reasonable
		e_object for verb found.  Used mostly by
		intransitive verbs.
*/
void CGameState::needobj()
{
	short wtype,wval;
										
	analyze(e_word1,&wtype,&wval, true);
	printf("%s what?\n",wtype == VOCAB_TYP_VRB ? e_word1 : e_word2);
}




/*
		Routine to take 1 Turn
*/
void CGameState::PlayGameTurn()
{
	short	i;
				
	++m_turns;
	
	if ( m_location != LOC_000_Limbo && ! ValidCaveLocation( m_location ) )
	{
		printf("PlayGameTurn() bad m_location(%d)\n", m_location);
		bug(BUG_PlayGameTurn_Cpp+21);
	}

	ShowMapFirst();
										
	/*
			if m_closing, then he can't leave except via
			the main office.
	*/
	if ( m_location != LOC_143_main_office )
	{
		if (m_newloc1 < LOC_009_below_grate && m_newloc1 != LOC_000_Limbo && m_closing) 
		{
			SpeakInfoMsg(MSG4_130_exit_closed_leave_via_main_office);
			m_newloc1 = m_location;
			if (!m_panic)
				m_clock2 = 15;
			m_panic = 1;
		}
	}
	/*
			see if a DWARF has seen him and has come
			from where he wants to go.
	*/
	if (m_newloc1 != m_location && !forced(m_location) && m_locCondition[m_location]&NOPIRAT  ==  0)
	{
		for (i = 1; i< (DWARFLAST); ++i)
		{
			if (m_dwarfOldLoc[i] == m_newloc1 && m_dwarfSeen[i])
			{
				m_newloc1  =  m_location;
				SpeakInfoMsg(MSG4_002_dwarf_knife_blocks_way);
				break;
			}
		}
	}
										
	DoDwarves();	/* & special DWARF(pirate who steals)	*/
										
	/* added by BDS C conversion */
	if (m_location != m_newloc1)
	{
		m_location = m_newloc1;
		
		/* check for death */
		if (m_location == LOC_000_Limbo) 
		{
			death();
			return;
		}
																				
		/* check for forced move */
		if (forced (m_location)) 
		{
			describe();
			domove();
			return;
		}
																				
		/* check for wandering in dark */
		if ( m_wasDark && dark() && pct(35, 10) ) 
		{
			SpeakInfoMsg(MSG4_023_fell_into_pit_broke_every_bone_body);
			m_oldloc2 = m_location;
			death();
			return;
		}
																				
		/* describe his situation */
		describe();
		if (!dark ()) 
		{
			++m_locVisitedCnt[m_location];
			descitem();
		}
																				
	}  /* if (m_location != m_newloc1)	*/
	else if ( e_motion )
	{
		/* describe his situation */
		describe();
		if (!dark ()) 
		{
			++m_locVisitedCnt[m_location];
			descitem();
		}
	}
													
	if (m_closed) 
	{
		if (m_objectProperty[OBJ_OYSTER] < 0 && toting (OBJ_OYSTER))
			SpeakObjDesc(OBJ_OYSTER, 1);

		for ( i = 1; i <= LastObjectIndexed(); ++i )
			if (toting (i) && m_objectProperty[i] < 0)
				m_objectProperty[i] = -1-m_objectProperty[i];
	}
										
	m_wasDark = dark();
	if (m_knifeLoc > LOC_000_Limbo && m_knifeLoc != m_location)
		m_knifeLoc  =  LOC_000_Limbo;
										
	if (stimer())	/* as the grains of sand slip by	*/
		return;
										
	DoHint(m_location);
										
	if ( Get_bAutoSave() &&  e_motion )
	{
		char* autoCpDir = strdup( "AutoCP" );
		char autoCpFileName[_MAX_PATH+1];
		mkdir( autoCpDir, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH );
		sprintf( autoCpFileName, "%s\\Turn_%04d_Loc_%03d", autoCpDir, m_turns, Get_location() );
		SaveAdv(autoCpFileName);
	}

	while (!english());	/* retrieve player instructions	*/
		
#if _DEBUG	
	if (g_debugFlg)
		printf("m_location = %d, e_verb = %d, e_object = %d, \
		motion = %d\n", m_location, e_verb, e_object, e_motion);
#endif
										
	if (e_motion)		/* execute player instructions	*/
		domove();
	else if (e_object)
			doobj();
	else
		itverb();
}


/*
		special time limit stuff...
*/
short CGameState::stimer()
{
	short	msg = MSG4_000_No_Message;
	short	i = 0;
										
	SaveRestoreTime( true );	// save stimer vars for possible restore
										
	m_foobar = m_foobar > 0 ?  -m_foobar : 0;
	if (m_tally1 == 0
		&& m_location >= LOC_015_hall_mists
		&& m_location != LOC_033_Y2
		)
		--m_clock1;
						
	if (m_clock1 == 0)
	{
		/*
				start m_closing the cave
		*/
		m_objectProperty[OBJ_GRATE] = 0;
		m_objectProperty[OBJ_FISSURE] = 0;
		for (i = 1; i<DWARFMAX; ++i)
			m_dwarfSeen[i] = 0;
		move(OBJ_TROLL, LOC_000_Limbo, LOC_000_MOVEABLE);
		move(OBJ_TROLL2, LOC_117_sw_chasm, LOC_122_ne_chasm);
		juggle(OBJ_CHASM);
		if (m_objectProperty[OBJ_BEAR] != 3)
			dstroy(OBJ_BEAR);
		m_objectProperty[OBJ_CHAIN] = 0;
		m_objectLoc2[OBJ_CHAIN] = LOC_000_Limbo;
		m_objectProperty[OBJ_AXE] = 0;
		m_objectLoc2[OBJ_AXE] = LOC_000_Limbo;

		Set_DwarfsEnabled(0);

		SpeakInfoMsg(MSG4_129_Cave_closing_soon_please_exit_now);
		m_clock1 = -1;
		m_closing = 1;
		return(0);
	}
	if (m_clock1 < 0)
		--m_clock2;
	if (m_clock2 == 0) 
	{
		/*
				set up storage room...
				and close the cave...
		*/
		m_objectProperty[OBJ_BOTTLE] = put(OBJ_BOTTLE, LOC_115_ne_end_repository, 1);
		m_objectProperty[OBJ_PLANT] = put(OBJ_PLANT, LOC_115_ne_end_repository, 0);
		m_objectProperty[OBJ_OYSTER] = put(OBJ_OYSTER, LOC_115_ne_end_repository, 0);
		m_objectProperty[OBJ_LAMP] = put(OBJ_LAMP, LOC_115_ne_end_repository, 0);
		m_objectProperty[OBJ_ROD] = put(OBJ_ROD, LOC_115_ne_end_repository, 0);
		m_objectProperty[OBJ_DWARF] = put(OBJ_DWARF, LOC_115_ne_end_repository, 0);

		m_oldloc1 = LOC_116_sw_end_repository;
		m_location = LOC_116_sw_end_repository;
		m_newloc1 = LOC_115_ne_end_repository;

		put(OBJ_GRATE, LOC_116_sw_end_repository, 0);

		m_objectProperty[OBJ_SNAKE] = put(OBJ_SNAKE, LOC_116_sw_end_repository, 1);
		m_objectProperty[OBJ_BIRD] = put(OBJ_BIRD, LOC_116_sw_end_repository, 1);
		m_objectProperty[OBJ_CAGE] = put(OBJ_CAGE, LOC_116_sw_end_repository, 0);
		m_objectProperty[OBJ_ROD2] = put(OBJ_ROD2, LOC_116_sw_end_repository, 0);
		m_objectProperty[OBJ_PILLOW] = put(OBJ_PILLOW, LOC_116_sw_end_repository, 0);
		m_objectProperty[OBJ_MIRROR] = put(OBJ_MIRROR, LOC_115_ne_end_repository, 0);
		m_objectLoc2[OBJ_MIRROR] = LOC_116_sw_end_repository;

		for ( i = 1; i <= LastObjectIndexed(); ++i )
			if (toting(i))
				dstroy(i);

		move(OBJ_KEYS, LOC_143_main_office, LOC_000_MOVEABLE);

		SpeakInfoMsg(MSG4_132_cave_now_closed);

		m_closed = 1;
		return(1);
	}
	if (m_objectProperty[OBJ_LAMP] == 1)
		--m_limit;
	if (m_limit <= 30 &&
		here(OBJ_BATTERIES) && m_objectProperty[OBJ_BATTERIES] == 0 &&
		here(OBJ_LAMP)) 
	{
		SpeakInfoMsg(MSG4_188_lamp_getting_dim_replacing_batteries);
		m_objectProperty[OBJ_BATTERIES] = 1;
		if (toting(OBJ_BATTERIES))
			drop(OBJ_BATTERIES, m_location);
		m_limit += 2500;
		m_lampWarning = 0;
		return(0);
	}
	if (m_limit == 0) 
	{
		--m_limit;
		m_objectProperty[OBJ_LAMP] = 0;
		if (here(OBJ_LAMP))
			SpeakInfoMsg(MSG4_184_lamp_has_run_out_power);
		return(0);
	}
	if (m_limit < 0 && m_location <= LOC_008_outside_grate) 
	{
		SpeakInfoMsg(MSG4_185_no_point_wandering_out_here_So_let_s_call);
		m_gaveup = 1;
		normend();
	}
	if (m_limit <= 30) 
	{
		if (m_lampWarning || !here(OBJ_LAMP))
			return(0);
		m_lampWarning = 1;
		msg = MSG4_187_lamp_getting_dim_go_back_for_batteries;
		if (m_objectLoc1[OBJ_BATTERIES] == LOC_000_Limbo)
			msg = MSG4_183_lamp_getting_dim_best_wrap_up_or_find_batteries;
		if (m_objectProperty[OBJ_BATTERIES] == 1)
			msg = MSG4_189_lamp_getting_dim_no_spare_batteries;
		SpeakInfoMsg(msg);
		return(0);
	}
	return(0);
}

/*
		Routine to handle motion requests
*/
void CGameState::domove()
{
	/*
			if object was entered, is object here?
	*/
	if ( e_object )
	{
		if (!(m_objectLoc2[e_object] == m_location || here(e_object)))
		{
			printf("I see no %s here.\n", probj(e_object));
			return;
		}
		e_object = OBJ_NONE;
	}

	CTravelArray	travel;
	gettrav( m_location, travel );

	switch(e_motion) 
	{
		case MOV_NOWHERE:
			e_motion = MOV_NONE;
			break;
		case MOV_BACK:
			goback(travel);
			break;
		case MOV_CAVE:
			if (m_location<LOC_008_outside_grate)
				SpeakInfoMsg(MSG4_057_don_t_know_where_cave_is_follow_stream);
			else
				SpeakInfoMsg(MSG4_058_need_more_detailed_instructions);
			e_motion = MOV_NONE;
			break;
		default:
			m_oldloc2 = m_oldloc1;
			m_oldloc1 = m_location;
			dotrav(travel);
	}
}


/*
		Routine to handle request to return
		from whence we came!
*/
void CGameState::goback( CTravelArray&	travel )
{
	short	kk, want;
	short	sav_newloc1 = m_newloc1;
										
	if (forced(m_oldloc1))
		want = m_oldloc2;
	else
		want = m_oldloc1;
	m_oldloc2 = m_oldloc1;
	m_oldloc1 = m_location;

	if (want == m_location) 
	{
		SpeakInfoMsg(MSG4_091_dont_remember_how_you_got_here);
		return;
	}

	for ( kk = 0; kk <= travel.MaxIndexed(); ++kk ) 
	{
		if ( ! travel.Tcond(kk ) )
		{
			if ( travel.Dest(kk) == want ) 
			{
				e_motion = travel.Verb(kk);
				dotrav( travel );
				return;
			}

			if ( ValidCaveLocation( travel.Dest(kk) ) )
			{
				if ( forced( travel.Dest(kk) ) ) 
				{
					CTravelArray ftrav;
					gettrav( travel.Dest(kk), ftrav );
					if ( ftrav.Dest(0) == want)
					{
						e_motion = ftrav.Verb(0);
						dotrav( ftrav );
						if ( m_newloc1 == want )
							return;
						m_newloc1 = sav_newloc1;
					}
				}
			}
		}
	}
	SpeakInfoMsg(MSG4_140_can_t_get_there_from_here);
}

/*
		Routine to figure out a new m_location
		given current m_location and a motion.
*/
void CGameState::dotrav( CTravelArray&	travel )
{
	short	mvflag, hitflag, kk;
	short	rdest, rverb, rcond, rObjOrPct;
	short	pctt;
										
	m_newloc1 = m_location;
	mvflag = hitflag = 0;
	pctt = Rrand(0, 99);
										
	for (kk = 0; kk <= travel.MaxIndexed() && !mvflag; ++kk) 
	{
		bool bRc = travel( kk, rverb, rdest, rcond, rObjOrPct );

		if ( !bRc )
			break;

#ifdef _DEBUG
		if (g_debugFlg)
			printf("rdest = %d, rverb = %d, r_Condition = %d, \
			robject = %d in dotrav\n", rdest, rverb, \
			rcond, rObjOrPct);
#endif
		if ((rverb != 1) && (rverb != e_motion) && !hitflag)
			continue;
		++hitflag;
		switch(rcond) 
		{
			case 0:
				if ((rObjOrPct == 0) || (pctt <= rObjOrPct))
					++mvflag;
				else
#ifdef _DEBUG
					puts("");
				if (rObjOrPct && g_debugFlg)
					printf("%% move %d %d\n",
					pctt, mvflag);
#endif
				break;
			case 1:
				if (rObjOrPct == 0)
					++mvflag;
				else if (toting(rObjOrPct))
						++mvflag;
				break;
			case 2:
				if (toting(rObjOrPct) || at(rObjOrPct))
					++mvflag;
				break;
			case 3:
			case 4:
			case 5:
			case 7:
				if (m_objectProperty[rObjOrPct] != (rcond)-3)
					++mvflag;
				break;
			default:
				printf("dotrav() bad rcond(%d)\n", rcond);
				bug(BUG_PlayGameTurn_Cpp+37);
				return;
		}
	}
	if (!mvflag)
		badmove();
	else if (rdest>500)
			SpeakInfoMsg(rdest-500);
	else if (rdest>300)
			specialMove(rdest);
	else 
	{
		m_newloc1 = rdest;
#ifdef _DEBUG
		if (g_debugFlg)
			printf("m_newloc1 in dotrav = %d\n", m_newloc1);
#endif
	}
}

/*
		The player tried a poor move option.
*/
void CGameState::badmove()
{
	short msg = MSG4_012_don_t_know_how_apply_that_here;
										
	if (e_motion >= MOV_EAST && e_motion <= MOV_NW)
		msg = MSG4_009_There_no_way_that_direction;
	if (e_motion == MOV_UP || e_motion == MOV_DOWN)
		msg = MSG4_009_There_no_way_that_direction;
	if (e_motion == MOV_FORWARD || e_motion == MOV_LEFT	|| e_motion == MOV_RIGHT)
		msg = MSG4_010_Use_compass_points_or_nearby_objects;
	if (e_motion == MOV_OUTSIDE || e_motion == MOV_INWARD)
		msg = MSG4_011_Use_compass_points_or_name_something;
	if (e_verb == VRB_FIND || e_verb == VRB_INVENTORY)
		msg = MSG4_059_only_tell_you_what_you_see;
	if (e_motion == MOV_XYZZY || e_motion == MOV_PLUGH)
		msg = MSG4_042_Nothing_happens;
	if (e_motion == MOV_CRAWL) msg = MSG4_080_Which_way;
	e_motion = MOV_NONE;
	SpeakInfoMsg(msg);
}

/*
		Routine to handle very special movement.
*/
void CGameState::specialMove(short rdest)
{
	switch(rdest-300) 
	{
		case 1:  /* plover movement via alcove */
			if (!m_holding || (m_holding == 1 && toting(OBJ_EMERALD)))
				// alcove to plover or plover to alcove
				m_newloc1 = (LOC_099_alcove+LOC_100_plover_room)-m_location;
			else
				SpeakInfoMsg(MSG4_117_Something_you_re_carrying_won_t_fit_through_tunnel);
			break;
		case 2:  /* trying to remove plover, bad route */
			drop(OBJ_EMERALD, m_location);
			SpeakInfoMsg (MSG4_054_OK);
			break;
		case 3:  /* OBJ_TROLL bridge */
			if (m_objectProperty[OBJ_TROLL] == 1) 
			{
				SpeakObjDesc(OBJ_TROLL, 1);
				m_objectProperty[OBJ_TROLL] = 0;
				move(OBJ_TROLL2, LOC_000_Limbo, LOC_000_MOVEABLE);
				move(OBJ_TROLL, LOC_117_sw_chasm, LOC_122_ne_chasm);
				juggle(OBJ_CHASM);
				m_newloc1 = m_location;
			}
			else 
			{
				m_newloc1 = (m_location == LOC_117_sw_chasm ? LOC_122_ne_chasm : LOC_117_sw_chasm);
				if (m_objectProperty[OBJ_TROLL] == 0)
					++m_objectProperty[OBJ_TROLL];
				if (!toting (OBJ_BEAR))
					return;
				SpeakInfoMsg(MSG4_162_as_you_reach_other_side_bridge_buckles);
				m_objectProperty[OBJ_CHASM] = 1;
				m_objectProperty[OBJ_TROLL] = 2;
				drop(OBJ_BEAR, m_newloc1);
				m_objectLoc2[OBJ_BEAR] = LOC_Neg1_FIXED;
				m_objectProperty[OBJ_BEAR] = 3;
				if (m_objectProperty[OBJ_SPICES]<0)
					++m_tally2;
				m_oldloc2 = m_newloc1;
				death();
			}
			break;
		default:
			printf("dotrav() bad rdest(%d)\n", rdest);
			bug(BUG_PlayGameTurn_Cpp+38);
			return;
	}
}

/*
		Routines to process intransitive verbs
*/
void CGameState::itverb()
{
	bool bVerbProcessed = true;	// assume verb is processed unless default.

	//	These verbs need an valid object specified.
	switch(e_verb) 
	{
		case VRB_MAP:
		case VRB_WAVE:
		case VRB_CALM:
		case VRB_RUB:
		case VRB_THROW:
		case VRB_FIND:
		case VRB_FEED:
		case VRB_BREAK:
		case VRB_WAKE:
			needobj();
			bVerbProcessed = true;
			break;

		default:
			bVerbProcessed = false;
			break;
	}

	if ( bVerbProcessed )
		return;

	bVerbProcessed = true;	// assume verb is processed unless default.

	//	These verbs can take an optional e_word2 ( usually not an object )
	switch(e_verb) 
	{
		case VRB_BLAST:	// just being nice by allowing blast whatever.
			vblast();
			break;

		case VRB_FEE:
		case VRB_FIE:
		case VRB_FOE:
		case VRB_FOO:
		case VRB_FUM:
			ivfoo(e_word1, e_verb);
			break;
		case VRB_HOURS:
			actspk(e_verb);
			break;
		case VRB_SAY:
			vsay();
			break;
		case VRB_WALK:
			actspk(e_verb);
			break;
		case VRB_NOTHING:
			SpeakInfoMsg(MSG4_054_OK);
			break;
		case VRB_BRIEF:
			Toggle_brief();
			break;
		case VRB_AUTOCP:
			ivAutoCheckPoint();
			break;
		case VRB_SUSPEND:
			SaveRestoreTime();
			SaveAdv( e_word2 );
			exitAdv(0);
			break;
		case VRB_CHECKPOINT:
			SaveRestoreTime();
			SaveAdv( e_word2 );
			break;
		case VRB_RESTORE:
			RestoreAdv( e_word2 );
			describe(describeTypeLong);
			descitem();
			break;
		case VRB_CRYPT:
			SaveRestoreTime();
			crypt();
			break;
		case VRB_RESET:
			ResetGameState( 0 );
			break;
		case VRB_LOG:
			ivLog();
			break;
		case VRB_FRIENDLY:
			ivFriendly();
			break;
		case VRB_HINTS:
			ivHints();
			break;
		case VRB_VOCABLENGTH:
			ivVocabLength();
			break;
		case VRB_INPUT:
			SetInputType( e_word2 );
			break;

		default:
			bVerbProcessed = false;
			break;
	}

	if ( bVerbProcessed )
		return;

	bVerbProcessed = true;	// assume verb is processed unless default.

	//	These verbs should have no e_word2 specified.

	if ( strlen( e_word2 ) > 0 )
	{
		printf("Sorry, I don't know how to %s %s?\n", e_word1, e_word2 );
		return;
	}

	switch(e_verb) 
	{
		case VRB_QUIT:
			ivquit();
			break;
		case VRB_SCORE:
			SaveRestoreTime();
			score();
			break;
		case VRB_DESCRIBE:
			ivDescribe();
			break;
		case VRB_INVENTORY:
			SaveRestoreTime();
			inventory();
			break;

		default:
			bVerbProcessed = false;
			break;
	}

	if ( bVerbProcessed )
			return;

	bVerbProcessed = true;	// assume verb is processed unless default.

	//	These verbs should have no e_word2 specified
	//	Unless it was an object but we wouldn't be here if e_word2 was an object.

	//	These verbs will try to aquire an object.

	switch(e_verb) 
	{
		case VRB_ON:
		case VRB_OFF:
		case VRB_POUR:
			trverb();
			break;

		case VRB_TAKE:
			ivtake( e_object );
			break;
		case VRB_DROP:
			ivdrop( e_object );
			break;

		case VRB_OPEN:
		case VRB_LOCK:
			ivopen( e_object );
			break;


		case VRB_KILL:
			ivkill( e_verb, e_object );
			break;
		case VRB_EAT:
			iveat( e_object );
			break;
		case VRB_DRINK:
			ivdrink( e_object );
			break;
		case VRB_READ:
			ivread( e_object );
			break;
		case VRB_FILL:
			ivfill( e_object );
			break;

		default:
			bVerbProcessed = false;
			break;
	}

	if ( bVerbProcessed )
		return;

	printf("This intransitive not implemented yet\n");
}

/*
		Routines to process intransitive verbs
*/



/*
		CARRY, TAKE etc.
*/
void CGameState::ivtake( short& object )
{
	short anobj,item;

	if ( strlen( e_word2 ) > 0 )
	{
		printf("I don't see any %s here.\n", e_word2 );
		return;
	}
										
	anobj = 0;
	for( item = 1; item <= LastObjectIndexed(); ++item) 
	{
		if(m_objectLoc1[item]==m_location) 
		{
			if(anobj != 0) 
			{
				needobj();
				return;
			}
			anobj = item;
		}
	}
	if( anobj==0 || (dwarfCheck() && m_dwarfFlag>=2) ) 
	{
		needobj();
		return;
	}
	object = anobj;
	vtake();
}

/*
		DROP etc.
*/
void CGameState::ivdrop( short& object )
{
	short anobj,item;

	if ( strlen( e_word2 ) > 0 )
	{
		printf("Sorry, you're not holding any %s\n", e_word2 );
		return;
	}

	if ( m_holding < 1 )
	{
		printf("You have nothing to %s\n", e_word1);
		return;
	}
	if ( m_holding > 1 )
	{
		needobj();
		return;
	}
										
	anobj = 0;
	for( item = 1; item <= m_objectLoc1.MaxIndexed();++item ) 
	{
		if( toting(item) ) 
		{
			anobj = item;
		}
	}
	if( anobj == 0 ) 
	{
		needobj();
		return;
	}
	object = anobj;
	vdrop();
}

/*
		OPEN, LOCK, UNLOCK
*/
void CGameState::ivopen( short& object )
{
	if ( strlen( e_word2 ) > 0 )
	{
		printf("I don't know how to do that to %s\n", e_word2 );
		return;
	}

	if(here(OBJ_CLAM))
		object=OBJ_CLAM;
	if(here(OBJ_OYSTER))
		object=OBJ_OYSTER;
	if(at(OBJ_DOOR))
		object=OBJ_DOOR;
	if(at(OBJ_GRATE))
		object=OBJ_GRATE;
	if(here(OBJ_CHAIN)) 
	{
		if(object != 0) 
		{
			needobj();
			return;
		}
		object=OBJ_CHAIN;
	}
	if(toting(OBJ_MAP))
		object=OBJ_MAP;
	if(object==0) 
	{
		SpeakInfoMsg(MSG4_028_nothing_here_to_lock);
		return;
	}
	vopen();
}

/*
		ATTACK, KILL etc
*/
void CGameState::ivkill( short verb, short& object )
{
	if ( strlen( e_word2 ) > 0 )
	{
		printf("How can I %s %s?\n", e_word1, e_word2 );
		return;
	}
	m_object1 = 0;
	if( dwarfCheck() && m_dwarfFlag >=2 )
		object=OBJ_DWARF;
	if( here(OBJ_SNAKE) )
		addobj(OBJ_SNAKE, object);
	if( at(OBJ_DRAGON) && m_objectProperty[OBJ_DRAGON]==0 )
		addobj(OBJ_DRAGON, object);
	if( at(OBJ_TROLL) )
		addobj(OBJ_TROLL, object);
	if( here(OBJ_BEAR) && m_objectProperty[OBJ_BEAR]==0 )
		addobj(OBJ_BEAR, object);
	if( m_object1 != 0 ) 
	{
		needobj();
		return;
	}
	if(object != OBJ_NONE) 
	{
		vkill();
		return;
	}
	if(here(OBJ_BIRD) && verb!= VRB_THROW)
		object=OBJ_BIRD;
	if(here(OBJ_CLAM) || here(OBJ_OYSTER))
		addobj(OBJ_CLAM, object);
	if(m_object1 != OBJ_NONE) 
	{
		needobj();
		return;
	}
	vkill();
}

/*
		EAT
*/
void CGameState::iveat( short& object )
{
	if ( strlen( e_word2 ) > 0 )
	{
		printf("%s tastes like chicken.\n",  e_word2 );
		return;
	}

	if(!here(OBJ_FOOD))
		needobj();
	else 
	{
		object=OBJ_FOOD;
		veat();
	}
}

/*
		DRINK
*/
void CGameState::ivdrink( short& object )
{
	if ( strlen( e_word2 ) > 0 )
	{
		printf("%s made me sick.\n",  e_word2 );
		return;
	}
	if(liqloc(m_location) != OBJ_WATER &&
		(liq()!= OBJ_WATER || !here(OBJ_BOTTLE)))
		needobj();
	else 
	{
		object=OBJ_WATER;
		vdrink();
	}
}

/*
		QUIT
*/
void CGameState::ivquit()
{
	if(m_gaveup=yes(MSG4_022_really_want_quit_now,MSG4_054_OK,MSG4_054_OK))
		normend();
}

/*
		FILL
*/

void CGameState::ivfill( short& object )
{
	if ( strlen( e_word2 ) > 0 )
	{
		printf("Sorry, I don't know how to %s %s?\n", e_word1, e_word2 );
		return;
	}
	if(!here(OBJ_BOTTLE))
		needobj();
	else 
	{
		object=OBJ_BOTTLE;
		vfill();
	}
}

/*
		Handle fee fie foe foo...
*/
void CGameState::ivfoo( char* e_word1, short verb )
{
	short msg = MSG4_042_Nothing_happens;
	short k = verb - VRB_FEE + 1;
										
	if ( strlen( e_word2 ) > 0 )
	{
		printf("%s %s doesn't work here\n", e_word1, e_word2 );
		return;
	}

	if (m_foobar != 1-k)
	{
		if (m_foobar != 0)
			msg = MSG4_151_can_t_you_read_start_over;
		SpeakInfoMsg(msg);
		return;
	}
	m_foobar = k;
	if (k != 4)
		return;
	m_foobar = 0;
	if (m_objectLoc1[OBJ_EGGS] == LOC_092_giant_room ||
		(toting(OBJ_EGGS)
		&& m_location == LOC_092_giant_room)) 
	{
		SpeakInfoMsg(msg);
		return;
	}
	if (m_objectLoc1[OBJ_EGGS] == LOC_000_Limbo
		&& m_objectLoc1[OBJ_TROLL] == LOC_000_Limbo
		&& m_objectProperty[OBJ_TROLL] == 0)
		m_objectProperty[OBJ_TROLL] = 1;
	if (here(OBJ_EGGS))
		k = 1;
	else if (m_location == LOC_092_giant_room)
			k = 0;
	else
		k = 2;
	move(OBJ_EGGS,LOC_092_giant_room, LOC_000_MOVEABLE);
	SpeakObjDesc(OBJ_EGGS,k);
	return;
}

/*
		read etc...
*/
void CGameState::ivread( short& object )
{
	if ( strlen( e_word2 ) > 0 )
	{
		printf("Sorry, I can't %s %s?\n", e_word1, e_word2 );
		return;
	}
	if (here(OBJ_MAP))
		object = OBJ_MAP;
	if (here(OBJ_MAGAZINE))
		object = OBJ_MAGAZINE;
	if (here(OBJ_TABLET))
		object = object*(LastObjectIndexed()+1) + OBJ_TABLET;
	if (here(OBJ_MESSAGE))
		object = object*(LastObjectIndexed()+1) + OBJ_MESSAGE;
	if (object > LastObjectIndexed() || object == OBJ_NONE || dark()) 
	{
		needobj();
		return;
	}
	vread();
}

/*
		INVENTORY 
*/
void CGameState::inventory()
{
	short msg = MSG4_098_You_re_not_carrying_anything;
											
	for ( short i=1; i <= LastObjectIndexed(); ++i ) 
	{
		if (i==OBJ_BEAR || !toting(i))
			continue;
		if (msg)
			SpeakInfoMsg(MSG4_099_You_are_currently_holding_following);
		msg = MSG4_000_No_Message;
		SpeakObjDesc(i,-1);
	}
	if (toting(OBJ_BEAR))
		msg = MSG4_141_followed_by_tame_bear;
	if (msg)
		SpeakInfoMsg(msg);
}

/*
		VRB_DESCRIBE
*/
void CGameState::ivDescribe()
{
	if (m_detail++<3)
		SpeakInfoMsg(MSG4_015_not_allowed_more_detail_will_repeat_description);
	describe(describeTypeLong);
	descitem();
}

/*
		VRB_LOG
*/
void CGameState::ivLog()
{
	if ( strlen( e_word2 ) == 1 && isdigit(*e_word2) )
	{
		short nLogLevelSpecified = atoi( e_word2 );
		if ( nLogLevelSpecified < 3 )
		{
			Set_LogLevel( nLogLevelSpecified );
			return;
		}
	}
	if ( strcasecmp( e_word2, "delete") == 0)
	{
		deleteLogFile();
	}
	else if ( strlen( e_word2 ) == 0 )
	{
		Toggle_LogLevel();
	}
	else
	{
		setLogFileName( e_word2 );
	}
	return;
}

/*
		VRB_AUTOCP
*/
void CGameState::ivAutoCheckPoint()
{
	short	nVal = atoi( e_word2 );

	if ( strlen( e_word2 ) == 0 )
		Toggle_bAutoSave();
	else
		Set_bAutoSave( nVal );
}

/*
		VRB_FRIENDLY
*/
void CGameState::ivFriendly()
{
	short	nVal = atoi( e_word2 );

	if ( strlen( e_word2 ) == 0 )
		Toggle_Friendly();
	else
		Set_Friendly( nVal );
}
/*
		VRB_HINTS
*/
void CGameState::ivHints()
{
	short	nVal = atoi( e_word2 );

	if ( strlen( e_word2 ) == 0 )
		Toggle_OfferHints();
	else
		Set_OfferHints( nVal );
}
/*
		VRB_VOCABLENGTH
*/
void CGameState::ivVocabLength()
{
	short nNewLenSpecified = std::min( 20, atoi( e_word2 ) );
	if ( nNewLenSpecified > 3 )
		Set_VocabCmpLen( nNewLenSpecified, true );
	else
		Toggle_VocabCmpLen();
}

/*
		ensure uniqueness as objects are searched
		out for an intransitive verb
*/
void CGameState::addobj( short obj,  short& object )
{
	if(m_object1 != OBJ_NONE)
		return;
	if(object != OBJ_NONE) 
	{
		m_object1 = -1;
		return;
	}
	object = obj;
}

/*
		Routine to request whether a hint is desired.
*/
void CGameState::tryhint (int imsg, short mask, int idx)
{
	if ( ! Get_OfferHints( false ) )
		return;

	puts ("\n");
	if (yes (imsg, MSG4_000_No_Message, MSG4_054_OK))
		if (yes (MSG4_087_Hint_Offer, imsg+1, MSG4_054_OK)) 
	{
		++m_hintTaken;
//		m_hintAvail &= ~mask;
	}
	m_hintAvail &= ~mask; // aw moved to here, once is all it is offered

//	m_hintAreaCnt[idx] = 0;	// aw removed because we're only offering each hint once per game, I don't like tellamarketers :)
}

/*
		Routine to check and process hints.
*/
void CGameState::DoHint(short loc)
{
	if ( ! Get_OfferHints( false ) )
		return;

	if (m_locCondition[loc]&m_hintAvail)
	{
		switch (m_locCondition[loc]&HINT_MASK)
		{
			case HINTF:
				++m_hintAreaCnt[HINTAREAF];
				if (m_hintAreaCnt[HINTAREAF]>20 && m_locVisitedCnt[LOC_008_outside_grate] == 0 /* never in yet */)
					tryhint (MSG4_056_trying_to_find_cave, HINTF, HINTAREAF);
				break;
			case HINTC:
				++m_hintAreaCnt[HINTAREAC];
				if (m_hintAreaCnt[HINTAREAC]>3 && m_objectProperty[OBJ_GRATE] == 0 && !toting(OBJ_KEYS))
					tryhint (MSG4_062_trying_get_into_cave, HINTC, HINTAREAC);
				break;
			case HINTB:
				++m_hintAreaCnt[HINTAREAB];
				if (m_hintAreaCnt[HINTAREAB]>4 && m_objectLoc1[OBJ_BIRD] == loc && toting(OBJ_ROD))
					tryhint (MSG4_018_trying_catch_bird, HINTB, HINTAREAB);
				break;
			case HINTS:
				++m_hintAreaCnt[HINTAREAS];
				if (m_hintAreaCnt[HINTAREAS]>5 && m_objectLoc1[OBJ_SNAKE] == loc && !toting(OBJ_BIRD))
					tryhint (MSG4_020_Are_you_trying_deal_with_snake, HINTS, HINTAREAS);
				break;
			case HINTM:
				++m_hintAreaCnt[HINTAREAM];
				if (m_hintAreaCnt[HINTAREAM]>15)
					tryhint (MSG4_176_need_help_getting_out_maze, HINTM, HINTAREAM);
				break;
			case HINTP:
				++m_hintAreaCnt[HINTAREAP];
				if (m_hintAreaCnt[HINTAREAP]>5 && m_objectLoc1[OBJ_EMERALD] == LOC_100_plover_room)		// was != ??? aw.
					tryhint (MSG4_178_trying_explore_beyond_plover_room, HINTP, HINTAREAP);
				break;
			case HINTW:
				++m_hintAreaCnt[HINTAREAW];
				if (m_hintAreaCnt[HINTAREAW]>15)
					tryhint (MSG4_180_need_help_getting_out_wits_end, HINTW, HINTAREAW);
				break;
			case HINTO:
#if 0
				++m_hintAreaCnt[HINTAREAO];
				if (m_hintAreaCnt[HINTAREAO]>15)
					tryhint (, HINTO, HINTAREAO);
#endif
				break;
		}
	}
	else
	{
		m_hintAreaCnt.ResetMemory();
	}
}




static char szLastCheckPointName[_MAX_PATH+1];


typedef struct
{
	short*	var;
	short	rep;
}
INT_VARS;

/*
		save adventure game varables
*/
bool CGameState::IntVars( int nPivNdx, short*& var,  short& rep )
{
	static	INT_VARS playIntVars[] = 
	{
		&	m_Version,1,
		&	m_newloc1,1,
		&	m_location,1,
		&	m_oldloc1,1,
		&	m_oldloc2,1,
																				
		&	m_object1,1,

		&	m_bPirateEnabled,1,
		&	m_bDwarfsEnabled,1,

		&	m_dwarfFlag,1,
		&	m_dwarfAltLoc,1,
		&	m_dwarfsKilled,1,
																				
		&	m_knifeLoc,1,
		&	m_chestLoc1,1,
		&	m_chestLoc2,1,
																				
		&	m_holding,1,
		&	m_holdMax,1,
		&	m_turns,1,
		&	m_wasDark,1,
		&	m_closing,1,
		&	m_closed,1,
		&	m_tally1,1,
		&	m_tally2,1,
		&	m_limit,1,
		&	m_lampWarning,1,
		&	m_clock1,1,
		&	m_clock2,1,
		&	m_panic,1,
																				
		&	m_bonus,1,
		&	m_numdie,1,
		&	m_foobar,1,
		&	m_gaveup,1,
		
		&	m_detail,1,
		&	m_brief,1,
		&	m_nFriendly,1,
		&	m_nVocabCmpLen,1,
		&	m_nLogLevel,1,
						
		&	m_nOfferHints,1,										
		&	m_hintTaken,1,
		&	m_hintAvail,1,

		&	m_bAutoSave,1,

		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		&	m_Reserved,1,	// = 0x1234; // we'll write a few of these to reserve space for new vars.
		
		NULL,0
	};
#define NUM_INT_VARS (sizeof(playIntVars) / sizeof(playIntVars[0]))

	_ASSERT(NUM_INT_VARS == 48 ); // if this fails adjust the number of reserved.


	if ( nPivNdx < 0 || nPivNdx >= NUM_INT_VARS )
	{
		static short nNumIntVars = NUM_INT_VARS;
		var = &nNumIntVars;
		rep = 1;

		return false;
	}
										
	var = playIntVars[nPivNdx].var;
	rep = playIntVars[nPivNdx].rep;
										
	return true;
}

char* advTempRestoreFile = strdup( "advTempRestoreAw1234.adv" );

/*
		save adventure game
*/
bool CGameState::SaveAdv( char* szFileName )
{
	FILE*	savefd;
	char	fileName[_MAX_PATH+1];
	char	*sptr;
	bool	bRc = true;
	short	rep = 0;
	short*	var = NULL;
										
	if ( szFileName && strlen(szFileName) > 0 )
	{
		strcpy(fileName, szFileName);
	}
	else
	{
		printf("What do you want to name the saved game? ");
//		scanf("%s", fileName);
		GetInputLine( fileName, sizeof(fileName)-1, false);
	}
										
	if (sptr = strchr(fileName, '.'))
		*sptr = '\0';		/* kill extension	*/
	if (strlen(fileName) > _MAX_PATH - 4)
		fileName[_MAX_PATH - 4] = '\0';
	strcat(fileName, ".adv");
										
	savefd = fopen(fileName, "wb");
	if (savefd == NULL) 
	{
		printf("Sorry, I can't create the file...%s\n", fileName);
		return false;
	}

	//	First lets write the count of IntVars to write.
	IntVars(  -1 ,  var,  rep );
	fwrite( var, sizeof(*var), rep, savefd);
	if ( ferror( savefd ) )
	{
		printf("Write error on save file...%s\n", fileName);
		bRc = false;
	}

	if ( bRc )
	{
		for ( int	nPiv = 0;  ; ++nPiv )
		{
			bRc = IntVars(  nPiv ,  var,  rep );
			if ( !bRc || var == NULL )
				break;
			fwrite( var, sizeof(*var), rep, savefd);
			if ( ferror( savefd ) )
			{
				printf("Write error on save file...%s\n", fileName);
				bRc = false;
				break;
			}
		}
	}

	if ( bRc )
		bRc = m_locVisitedCnt.Save(savefd);
//	if ( bRc )
//		bRc = m_locCondition.Save(savefd);		// never changed during the game
																				
	if ( bRc )
		bRc = m_objectLoc1.Save(savefd);
	if ( bRc )
		bRc = m_objectLoc2.Save(savefd);
	if ( bRc )
		bRc = m_objectProperty.Save(savefd);

	if ( bRc )
		bRc = m_dwarfLoc1.Save(savefd);
	if ( bRc )
		bRc = m_dwarfSeen.Save(savefd);
	if ( bRc )
		bRc = m_dwarfOldLoc.Save(savefd);

	if ( bRc )
		bRc = m_hintAreaCnt.Save(savefd);


	if (fclose(savefd)  ==  -1) 
	{
		printf("Sorry, I can't close the file...%s\n", \
		fileName);
		bRc = false;
	}
	if ( bRc )
		SpeakInfoMsg(MSG4_054_OK);
										
	if ( bRc )
	{
		if ( strcasecmp( szFileName, advTempRestoreFile ) != 0 )
			strcpy( szLastCheckPointName, fileName );
	}
	else
	{
		printf("Save Failed!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
		unlink( fileName );
	}
	return bRc;
}

/*
		restore saved game handler
*/
bool CGameState::RestoreAdv( char* szFileName )
{
	FILE*	restfd;
	char	fileName[_MAX_PATH+1];
	char	*sptr;
	short	rep = 0;
	short*	var = NULL;
	bool	bRc = true;

	
	if ( szFileName && strlen(szFileName) > 0 )
	{
		strcpy(fileName, szFileName);
	}
	else
	{
		if ( strlen( szLastCheckPointName) > 0 )
		{
			strcpy( fileName, szLastCheckPointName );
		}
		else
		{
			printf("What is the name of the saved game? ");
//			scanf("%s", fileName);
			GetInputLine( fileName, sizeof(fileName)-1, false);
		}
	}
										
	if (sptr = strchr(fileName, '.'))
		*sptr = '\0';		/* kill extension	*/
	if (strlen(fileName) > _MAX_PATH - 4)
		fileName[_MAX_PATH - 4] = '\0';
	strcat(fileName, ".adv");

	restfd = fopen(fileName, "rb");
	if (restfd == NULL)
	{
		printf("Sorry, I can't open the file...%s\n", fileName);
		return false;
	}

	// lets save a copy to restore if we fail.
	if ( strcasecmp( szFileName, advTempRestoreFile ) != 0 )
	{
		if ( ! SaveAdv( advTempRestoreFile ) )
		{
			printf("Failed to save current state.\n");
			printf("If the restore fails, it will be fatal.\n");
			if ( ! yes(MSG4_230_Continue) )
				return false;
		}
	}

	//  First lets get the count of InVars we need to read.
	IntVars(  -1 ,  var,  rep );
	short nNumIntVarsToRead = *var;

	//	Next lets read the count of IntVars to written.
	fread( var, sizeof(*var), rep, restfd );
	if ( ferror( restfd ) )
	{
		printf("Read error on save file...%s\n", fileName);
		bRc = false;
	}
	if ( bRc )
	{
		if ( nNumIntVarsToRead != *var )
		{
			printf("Save file...%s has (%d) IntVars vs (%d) IntVars Expected\n", fileName, nNumIntVarsToRead ,*var );
			printf("Can't restore! file...%s\n", fileName);
			bRc = false;
		}
	}
		
	if ( bRc )
	{
		for ( int	nPiv = 0;  ; ++nPiv )
		{
			bRc = IntVars(  nPiv ,  var,  rep );
			if ( !bRc || var == NULL )
				break;
			fread(var, sizeof(*var), rep, restfd);
			if ( ferror( restfd ) )
			{
				printf("Read error on save file...%s\n", fileName);
				bRc = false;
				break;
			}
		}
	}

	if ( bRc )
		bRc = m_locVisitedCnt.Restore(restfd);
//	if ( bRc )
//		bRc = m_locCondition.Restore(restfd);		// never changed during the game
																				
	if ( bRc )
		bRc = m_objectLoc1.Restore(restfd);
	if ( bRc )
		bRc = m_objectLoc2.Restore(restfd);
	if ( bRc )
		bRc = m_objectProperty.Restore(restfd);

	if ( bRc )
		bRc = m_dwarfLoc1.Restore(restfd);
	if ( bRc )
		bRc = m_dwarfSeen.Restore(restfd);
	if ( bRc )
		bRc = m_dwarfOldLoc.Restore(restfd);

	if ( bRc )
		bRc = m_hintAreaCnt.Restore(restfd);

	if (fclose(restfd)  ==  -1)
	{
		printf("Warning -- can't close save file...%s\n", \
		fileName);
		bRc = false;
	}

	if ( strcasecmp( szFileName, advTempRestoreFile ) == 0 )
	{
		unlink ( advTempRestoreFile );
		if ( bRc )
		{
			SpeakInfoMsg(MSG4_054_OK);
		}
		else
		{
			printf("Undo Failed! *** Fatal ***\n");
			bug(BUG_GameStateSaveRestore_cpp+501);  // bad restore
		}
	}
	else
	{
		if ( bRc )
		{
			unlink ( advTempRestoreFile );
			SpeakInfoMsg(MSG4_054_OK);
		}
		else
		{
			printf("Restore Failed! --- Trying Undo ---\n");
			RestoreAdv( advTempRestoreFile );
		}
	}
	return bRc;
}



/*
		Initialization of adventure play variables
*/
void CGameState::initplay()
{
	m_Version = 1003;			/* Program Version Should match RC */

	InitCaveTable();						
	InitPlayLocationTables();
	InitPlayObjectTables();

	m_newloc1 = LOC_001_end_road;
	m_location = LOC_001_end_road+1;
	m_oldloc1 = LOC_001_end_road+1;
	m_oldloc2 = LOC_001_end_road+1;
						
	m_dwarfSeen.ResetMemory();
	m_dwarfOldLoc.ResetMemory();
	
	short nIdx = 0;
	m_dwarfLoc1[nIdx++] = LOC_000_Limbo;
	m_dwarfLoc1[nIdx++] = LOC_019_hall_mt_king;
	m_dwarfLoc1[nIdx++] = LOC_027_west_fissure_hall_mists;
	m_dwarfLoc1[nIdx++] = LOC_033_Y2;
	m_dwarfLoc1[nIdx++] = LOC_044_maze;
	m_dwarfLoc1[nIdx++] = LOC_063_Dead_end;
	m_dwarfLoc1[nIdx++] = LOC_114_Dead_end;
	
	m_bPirateEnabled = 1;	//  0 Pirate not allowed
	m_bDwarfsEnabled = 1;	//  0 Dwarfs not allowed
	
	m_dwarfFlag = 0;
	m_dwarfsKilled = 0;
	m_dwarfAltLoc = LOC_018_nugget_gold_room;
	
	m_knifeLoc = LOC_000_Limbo;
	m_chestLoc1 = LOC_114_Dead_end;
	m_chestLoc2 = LOC_140_Dead_end;
	
	m_holding = 0;
	m_holdMax = 7;
	m_turns = 0;
	m_wasDark = 0;
	m_closed = 0;
	m_closing = 0;
	m_tally1 = 15;			// number of treasures dec for each one found
	m_tally2 = 0;			// inc if bird dies before snake.
							// inc for each toll bridge before finding spice.
							// when tally1 meets tally2, time to close cave is near.
	m_limit = 100;
	m_lampWarning = 0;
	m_clock1 = 30;
	m_clock2 = 50;
	m_panic = 0;
	
	m_bonus = 0;
	m_numdie = 0;
	m_foobar = 0;
	m_gaveup = 0;
						
	m_detail = 0;
	m_brief = 1;	// short descriptions after visited
	m_nFriendly = 0;
	m_nLogLevel = 2;
	m_nVocabCmpLen = 20;	// set to 5 to be like orignal game
//	m_szLogFileName = "AdventureCCaw1_log.txt";
							
	m_nOfferHints = 1;
	m_hintTaken = 0;
	m_hintAvail = HINT_MASK;
	m_hintAreaCnt.ResetMemory();
	
	m_bAutoSave = 0;		// 0 is no auto save.
	m_Reserved = 0x1234;	// we'll write a few of these to reserve space for new vars.

	m_bMapOpen = 0;		// no need to save and restore.
	m_bNoDieTest = 0;	// no need to save and restore.
						
	return;
}



/*
		Initialization of adventure play location condition
*/
void CGameState::InitPlayLocationCondtion(short loc, short cond)
{
	m_locCondition[loc] = cond;
}

/*
		Initialization of adventure play location variables
*/
void CGameState::InitPlayLocationTables()
{
	/* initialize visited locations */
	m_locVisitedCnt.ResetMemory();				// 0 is not visited
	
	/* initialize location status array */
	m_locCondition.ResetMemory();


	return;	// this is now read from ADVENT0.

#if 0
	// Note: All locations are dark unless condition indicates LIGHT.				
	InitPlayLocationCondtion( LOC_000_Limbo,				0 );
	InitPlayLocationCondtion( LOC_001_end_road,				HINTF+LIQUID+LIGHT );
	InitPlayLocationCondtion( LOC_002_hill_road,			HINTF+LIGHT );
	InitPlayLocationCondtion( LOC_003_well_house,			HINTF+LIQUID+LIGHT );
	InitPlayLocationCondtion( LOC_004_valley,				HINTF+LIQUID+LIGHT );
	InitPlayLocationCondtion( LOC_005_forest,				HINTF+LIGHT );
	InitPlayLocationCondtion( LOC_006_forest,				HINTF+LIGHT );
	InitPlayLocationCondtion( LOC_007_slit_streambed,		HINTF+LIQUID+LIGHT );
	InitPlayLocationCondtion( LOC_008_outside_grate,		HINTC+LIGHT );
	InitPlayLocationCondtion( LOC_009_below_grate,			LIGHT );
	InitPlayLocationCondtion( LOC_010_cobble_crawl,			LIGHT );
	InitPlayLocationCondtion( LOC_011_debris_room,		0 );
	InitPlayLocationCondtion( LOC_012_EW_canyon,		0 );
	InitPlayLocationCondtion( LOC_013_bird_chamber,			HINTB );
	InitPlayLocationCondtion( LOC_014_top_pit,			0 );
	InitPlayLocationCondtion( LOC_015_hall_mists,		0 );
	InitPlayLocationCondtion( LOC_016_crack,				FORCED_MOVE );
	InitPlayLocationCondtion( LOC_017_east_bank_fissure,	0 );
	InitPlayLocationCondtion( LOC_018_nugget_gold_room,	0 );
	InitPlayLocationCondtion( LOC_019_hall_mt_king,			HINTS );
	InitPlayLocationCondtion( LOC_020_bottom_pit,			FORCED_MOVE );
	InitPlayLocationCondtion( LOC_021_didn_t_make_it,		FORCED_MOVE );
	InitPlayLocationCondtion( LOC_022_dome_unclimbable,		FORCED_MOVE );
	InitPlayLocationCondtion( LOC_023_west_end_twopit_room,	0 );
	InitPlayLocationCondtion( LOC_024_east_pit,				LIQUID+OIL );
	InitPlayLocationCondtion( LOC_025_west_pit,			0 );
	InitPlayLocationCondtion( LOC_026_clamber_plant,		FORCED_MOVE );
	InitPlayLocationCondtion( LOC_027_west_fissure_hall_mists,	0 );
	InitPlayLocationCondtion( LOC_028_low_NS_passage,	0 );
	InitPlayLocationCondtion( LOC_029_S_side_chamber,	0 );
	InitPlayLocationCondtion( LOC_030_E_chamber_hall_mountain_king,	0 );
	InitPlayLocationCondtion( LOC_031_GT_DOLLAR_LT,			FORCED_MOVE );
	InitPlayLocationCondtion( LOC_032_can_t_get_by_snake,	FORCED_MOVE );
	InitPlayLocationCondtion( LOC_033_Y2,				0 );
	InitPlayLocationCondtion( LOC_034_jumble_rock,		0 );
	InitPlayLocationCondtion( LOC_035_window_pit,		0 );
	InitPlayLocationCondtion( LOC_036_dirty_passage,	0 );
	InitPlayLocationCondtion( LOC_037_climbable_pit,	0 );
	InitPlayLocationCondtion( LOC_038_bottom_pit_stream,	LIQUID );
	InitPlayLocationCondtion( LOC_039_rock_room,		0 );
	InitPlayLocationCondtion( LOC_040_low_passage_p_n_hall_mists,	FORCED_MOVE );
	InitPlayLocationCondtion( LOC_041_west_end_hall_mists,	0 );
	InitPlayLocationCondtion( LOC_042_maze,					HINTM );
	InitPlayLocationCondtion( LOC_043_maze,					HINTM );
	InitPlayLocationCondtion( LOC_044_maze,					HINTM );
	InitPlayLocationCondtion( LOC_045_maze,					HINTM );
	InitPlayLocationCondtion( LOC_046_Dead_end,				HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_047_Dead_end,				HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_048_Dead_end,				HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_049_maze,					HINTM );
	InitPlayLocationCondtion( LOC_050_maze,					HINTM );
	InitPlayLocationCondtion( LOC_051_maze,					HINTM );
	InitPlayLocationCondtion( LOC_052_maze,					HINTM );
	InitPlayLocationCondtion( LOC_053_maze,					HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_054_Dead_end,				HINTM );
	InitPlayLocationCondtion( LOC_055_maze,					HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_056_Dead_end,			0 );
	InitPlayLocationCondtion( LOC_057_brink_of_pit,			NOPIRAT );
	InitPlayLocationCondtion( LOC_058_Dead_end,			0 );
	InitPlayLocationCondtion( LOC_059_low_passage_p_n_hall_mists,	FORCED_MOVE );
	InitPlayLocationCondtion( LOC_060_E_end_long_hall,	0 );
	InitPlayLocationCondtion( LOC_061_W_end_long_hall,	0 );
	InitPlayLocationCondtion( LOC_062_crossover_high_NS_passage_low_EW_one,	0 );
	InitPlayLocationCondtion( LOC_063_Dead_end,			0 );
	InitPlayLocationCondtion( LOC_064_complex_junction,	0 );
	InitPlayLocationCondtion( LOC_065_bedquilt_passage,	0 );
	InitPlayLocationCondtion( LOC_066_swiss_cheese_room,	0 );
	InitPlayLocationCondtion( LOC_067_east_end_twopit_room,	0 );
	InitPlayLocationCondtion( LOC_068_slab_room,		0 );
	InitPlayLocationCondtion( LOC_069_secret_NS_canyon_above_large_room,	0 );
	InitPlayLocationCondtion( LOC_070_secret_NS_canyon_above_sizable_passage,	0 );
	InitPlayLocationCondtion( LOC_071_junction_three_secret_canyons,	0 );
	InitPlayLocationCondtion( LOC_072_large_low_room,	0 );
	InitPlayLocationCondtion( LOC_073_Dead_end_crawl,	0 );
	InitPlayLocationCondtion( LOC_074_secret_EW_canyon,	0 );
	InitPlayLocationCondtion( LOC_075_wide_place_NS_canyon,	0 );
	InitPlayLocationCondtion( LOC_076_too_tight_canyon,	0 );
	InitPlayLocationCondtion( LOC_077_tall_EW_canyon,	0 );
	InitPlayLocationCondtion( LOC_078_canyon_dead_end,	0 );
	InitPlayLocationCondtion( LOC_079_stream_flows_out_pair_1_foot_diameter_pipes,	OIL+LIGHT );
	InitPlayLocationCondtion( LOC_080_maze,					HINTM );
	InitPlayLocationCondtion( LOC_081_Dead_end,				HINTM );
	InitPlayLocationCondtion( LOC_082_Dead_end,				HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_083_maze,				0 );
	InitPlayLocationCondtion( LOC_084_maze,				0 );
	InitPlayLocationCondtion( LOC_085_Dead_end,				NOPIRAT );
	InitPlayLocationCondtion( LOC_086_Dead_end,				HINTM+NOPIRAT );
	InitPlayLocationCondtion( LOC_087_maze,					HINTM );
	InitPlayLocationCondtion( LOC_088_narrow_corridor,	0 );
	InitPlayLocationCondtion( LOC_089_Use_up_or_out_leave_pit,	FORCED_MOVE );
	InitPlayLocationCondtion( LOC_090_climbed_plant_out_of_pit,	FORCED_MOVE );
	InitPlayLocationCondtion( LOC_091_steep_incline_above_large_room,	0 );
	InitPlayLocationCondtion( LOC_092_giant_room,		0 );
	InitPlayLocationCondtion( LOC_093_passage_here_blocked_by_recent_cave_in,	0 );
	InitPlayLocationCondtion( LOC_094_one_end_northsouth_passage,	0 );
	InitPlayLocationCondtion( LOC_095_cavern_waterfall,		LIQUID );
	InitPlayLocationCondtion( LOC_096_soft_room,		0 );
	InitPlayLocationCondtion( LOC_097_oriental_room,	0 );
	InitPlayLocationCondtion( LOC_098_misty_cavern,		0 );
	InitPlayLocationCondtion( LOC_099_alcove,				HINTP );
	InitPlayLocationCondtion( LOC_100_plover_room,			HINTP+LIGHT );
	InitPlayLocationCondtion( LOC_101_dark_room,		0 );
	InitPlayLocationCondtion( LOC_102_arched_hall,		0 );
	InitPlayLocationCondtion( LOC_103_shell_room,		0 );
	InitPlayLocationCondtion( LOC_104_corridor_ragged_sharp_walls,	0 );
	InitPlayLocationCondtion( LOC_105_cul_de_sac,		0 );
	InitPlayLocationCondtion( LOC_106_anteroom,			0 );
	InitPlayLocationCondtion( LOC_107_maze,				0 );
	InitPlayLocationCondtion( LOC_108_Witt_s_end,			HINTW );
	InitPlayLocationCondtion( LOC_109_mirror_canyon,	0 );
	InitPlayLocationCondtion( LOC_110_window_pit,		0 );
	InitPlayLocationCondtion( LOC_111_top_stalactite,	0 );
	InitPlayLocationCondtion( LOC_112_maze,				0 );
	InitPlayLocationCondtion( LOC_113_reservoir,			LIQUID );
	InitPlayLocationCondtion( LOC_114_Dead_end,			0 );
	InitPlayLocationCondtion( LOC_115_ne_end_repository,	LIGHT );
	InitPlayLocationCondtion( LOC_116_sw_end_repository,	LIGHT );
	InitPlayLocationCondtion( LOC_117_sw_chasm,			0 );
	InitPlayLocationCondtion( LOC_118_sloping_corridor,	0 );
	InitPlayLocationCondtion( LOC_119_secret_canyon,	0 );
	InitPlayLocationCondtion( LOC_120_secret_canyon,	0 );
	InitPlayLocationCondtion( LOC_121_secret_canyon,	0 );
	InitPlayLocationCondtion( LOC_122_ne_chasm,				NOPIRAT );
	InitPlayLocationCondtion( LOC_123_corridor,				NOPIRAT );
	InitPlayLocationCondtion( LOC_124_fork_path,			NOPIRAT );
	InitPlayLocationCondtion( LOC_125_junction_warm_walls,	NOPIRAT );
	InitPlayLocationCondtion( LOC_126_breath_taking_view,	NOPIRAT );
	InitPlayLocationCondtion( LOC_127_chamber_boulders,		NOPIRAT );
	InitPlayLocationCondtion( LOC_128_limestone_passage,	NOPIRAT );
	InitPlayLocationCondtion( LOC_129_front_barren_room,	NOPIRAT );
	InitPlayLocationCondtion( LOC_130_barren_room,			NOPIRAT );
	InitPlayLocationCondtion( LOC_131_maze,				0 );
	InitPlayLocationCondtion( LOC_132_maze,				0 );
	InitPlayLocationCondtion( LOC_133_maze,				0 );
	InitPlayLocationCondtion( LOC_134_maze,				0 );
	InitPlayLocationCondtion( LOC_135_maze,				0 );
	InitPlayLocationCondtion( LOC_136_maze,				0 );
	InitPlayLocationCondtion( LOC_137_maze,				0 );
	InitPlayLocationCondtion( LOC_138_maze,				0 );
	InitPlayLocationCondtion( LOC_139_maze,				0 );
	InitPlayLocationCondtion( LOC_141_forest,				LIGHT+NOPIRAT );
	InitPlayLocationCondtion( LOC_142_tree_house,			LIGHT+NOPIRAT );
	InitPlayLocationCondtion( LOC_143_main_office,			LIGHT+NOPIRAT );
#endif

}

/*
		Initialization of adventure play Object
*/
void CGameState::	InitPlayObjectLoc(short obj, short loc1, short loc2)
{
	m_objectLoc1[obj] = loc1;
	m_objectLoc2[obj] = loc2;

	if ( obj >= OBJ_NUGGET )
		m_objectProperty[obj] = -1;	// just in case someone adds treasure > OBJ_LAST_TREASURE
}

/*
		Initialization of adventure play Object variables
*/
void CGameState::InitPlayObjectTables()
{
	_ASSERT(OBJ_LAST_TREASURE < 100 /*MAXOBJ*/);

	//	0 is normal but can have different meaning for each object(item).
	m_objectProperty.ResetMemory();

	// -1 is treasure not found
	for (short obj = FirstTreasure(); obj <= LastTreasure(); ++obj)
		m_objectProperty[obj] = -1;

	m_objectLoc1.ResetMemory();		// normal or fixed object location
	m_objectLoc2.ResetMemory();		// 0 is normal (get-able), not 0 is fixed or not get-able  object
	
	return;	// this is now read from ADVENT0.

#if 0
	InitPlayObjectLoc(OBJ_KEYS,			LOC_003_well_house, 0);
	InitPlayObjectLoc(OBJ_LAMP,			LOC_003_well_house, 0);
	InitPlayObjectLoc(OBJ_GRATE,		LOC_008_outside_grate,				LOC_009_below_grate);
	InitPlayObjectLoc(OBJ_CAGE,			LOC_010_cobble_crawl, 0);
	InitPlayObjectLoc(OBJ_ROD,			LOC_011_debris_room, 0);
	InitPlayObjectLoc(OBJ_ROD2,			LOC_000_Limbo, 0);	// dynamite in repository
	InitPlayObjectLoc(OBJ_STEPS,		LOC_014_top_pit,					LOC_015_hall_mists);
	InitPlayObjectLoc(OBJ_BIRD,			LOC_013_bird_chamber, 0);
	InitPlayObjectLoc(OBJ_DOOR,			LOC_094_one_end_northsouth_passage, -1);
	InitPlayObjectLoc(OBJ_PILLOW,		LOC_096_soft_room, 0);
	InitPlayObjectLoc(OBJ_SNAKE,		LOC_019_hall_mt_king,			-1);
	InitPlayObjectLoc(OBJ_FISSURE,		LOC_017_east_bank_fissure,			LOC_027_west_fissure_hall_mists);
	InitPlayObjectLoc(OBJ_TABLET,		LOC_101_dark_room,				-1);
	InitPlayObjectLoc(OBJ_CLAM,			LOC_103_shell_room, -1);
	InitPlayObjectLoc(OBJ_OYSTER,		LOC_000_Limbo, 0);	// replaces clam after clam opened
	InitPlayObjectLoc(OBJ_MAGAZINE,		LOC_106_anteroom, 0);
	InitPlayObjectLoc(OBJ_DWARF,		LOC_000_Limbo,					-1);
	InitPlayObjectLoc(OBJ_KNIFE,		LOC_000_Limbo,	0);
	InitPlayObjectLoc(OBJ_FOOD,			LOC_003_well_house,	0);
	InitPlayObjectLoc(OBJ_BOTTLE,		LOC_003_well_house,	0);
	InitPlayObjectLoc(OBJ_MIRROR,		LOC_109_mirror_canyon,			-1);
	InitPlayObjectLoc(OBJ_PLANT,		LOC_025_west_pit,				-1);
	InitPlayObjectLoc(OBJ_PLANT2,		LOC_023_west_end_twopit_room,		LOC_067_east_end_twopit_room);
	InitPlayObjectLoc(OBJ_STALACTITE,	LOC_111_top_stalactite,			-1);
	InitPlayObjectLoc(OBJ_FIGURE,		LOC_035_window_pit,					LOC_110_window_pit);
	InitPlayObjectLoc(OBJ_AXE,			LOC_000_Limbo, 0);	
	InitPlayObjectLoc(OBJ_DRAWING,		LOC_097_oriental_room,			-1);
	InitPlayObjectLoc(OBJ_DRAGON,		LOC_119_secret_canyon,				LOC_121_secret_canyon);
	InitPlayObjectLoc(OBJ_CHASM,		LOC_117_sw_chasm,					LOC_122_ne_chasm);
	InitPlayObjectLoc(OBJ_TROLL,		LOC_117_sw_chasm,					LOC_122_ne_chasm);
	InitPlayObjectLoc(OBJ_TROLL2,		LOC_000_Limbo, 0);	// replaces first troll
	InitPlayObjectLoc(OBJ_BEAR,			LOC_130_barren_room,			-1);
	InitPlayObjectLoc(OBJ_MESSAGE,		LOC_000_Limbo, -1);
	InitPlayObjectLoc(OBJ_VOLCANO,		LOC_126_breath_taking_view,		-1);
	InitPlayObjectLoc(OBJ_VENDING,		LOC_140_Dead_end,				-1);
	InitPlayObjectLoc(OBJ_BATTERIES,	LOC_000_Limbo, 0);	// inside vending machine
	InitPlayObjectLoc(OBJ_CARPET,		LOC_096_soft_room,				-1);
	InitPlayObjectLoc(OBJ_MAP,			LOC_142_tree_house,	0);
	InitPlayObjectLoc(OBJ_TREE,			LOC_141_forest,					-1);

//	Treasures have a value >= OBJ_NUGGET  and <= OBJ_LAST_TREASURE

	InitPlayObjectLoc(OBJ_NUGGET,		LOC_018_nugget_gold_room, 0);
	InitPlayObjectLoc(OBJ_DIAMONDS,		LOC_027_west_fissure_hall_mists, 0);
	InitPlayObjectLoc(OBJ_SILVER,		LOC_028_low_NS_passage, 0);
	InitPlayObjectLoc(OBJ_JEWELS,		LOC_029_S_side_chamber, 0);
	InitPlayObjectLoc(OBJ_COINS,		LOC_030_E_chamber_hall_mountain_king, 0);
	InitPlayObjectLoc(OBJ_CHEST,		LOC_000_Limbo, 0);
	InitPlayObjectLoc(OBJ_EGGS,			LOC_092_giant_room, 0);
	InitPlayObjectLoc(OBJ_TRIDENT,		LOC_095_cavern_waterfall, 0);
	InitPlayObjectLoc(OBJ_VASE,			LOC_097_oriental_room, 0);	// need pillow to drop on
	InitPlayObjectLoc(OBJ_EMERALD,		LOC_100_plover_room, 0);
	InitPlayObjectLoc(OBJ_PYRAMID,		LOC_101_dark_room, 0);
	InitPlayObjectLoc(OBJ_PEARL,		LOC_000_Limbo, 0);	// inside clam, need trident to get.
	InitPlayObjectLoc(OBJ_RUG,			LOC_119_secret_canyon,				LOC_121_secret_canyon);
	InitPlayObjectLoc(OBJ_SPICES,		LOC_127_chamber_boulders,	0);
	InitPlayObjectLoc(OBJ_CHAIN,		LOC_130_barren_room, 0);
#endif

}


short ConvertLocCondStringToShort( char* locCondStr )
{
/*
	BIT mapping of "m_locCondition" array which indicates location status
*/
	char word[20];
	short ii = 0;
	
	while ( ii < sizeof(word) )
	{
		char c = locCondStr[ii];
		if ( c > ' ' && c != '|' )
			word[ii++] = c;
		else
			break;
	}
	word[ii] = '\0';

	if ( strcasecmp( "DARK", word ) == 0 )
		return DARK;
	if ( strcasecmp( "LIGHT", word ) == 0 )
		return LIGHT;
	if ( strcasecmp( "OIL", word ) == 0 )
		return OIL;
	if ( strcasecmp( "LIQUID", word ) == 0 )
		return LIQUID;
	if ( strcasecmp( "NOPIRAT", word ) == 0 )
		return NOPIRAT;
	if ( strcasecmp( "FORCED_MOVE", word ) == 0 )
		return FORCED_MOVE;
	if ( strcasecmp( "HINTC", word ) == 0 )
		return HINTC;
	if ( strcasecmp( "HINTB", word ) == 0 )
		return HINTB;
	if ( strcasecmp( "HINTS", word ) == 0 )
		return HINTS;
	if ( strcasecmp( "HINTM", word ) == 0 )
		return HINTM;
	if ( strcasecmp( "HINTP", word ) == 0 )
		return HINTP;
	if ( strcasecmp( "HINTW", word ) == 0 )
		return HINTW;
	if ( strcasecmp( "HINTF", word ) == 0 )
		return HINTF;
	if ( strcasecmp( "HINTO", word ) == 0 )
		return HINTO;

	return 0;
}

/*
		Routine to fill one State table entry
*/
bool	CGameState::LoadStateTableEntry( char* lineIn )
{
	bool bRc = true;

	if ( lineIn[0] != '!' )
		return bRc;

	int nLoc = atoi(&lineIn[1]);

	if ( nLoc == 0 )
		return bRc;

	if ( nLoc < 1 )
	{
		printf ( "Location (%d) in (%s) < 1\n", nLoc, lineIn );
		bug(BUG_AdvMessageData_Cpp+3);
	}

	bool	bContinue = true;
	char*	pDelim = lineIn;
	char	delim = '|';
	int		numDelims = 0;
	short	nSeq = 0;
	short	nLocCondition = 0;
	bool	bLocCondProcess = false;

	short	nObj,	nLoc1, nLoc2 = 0;
	bool	bObjLocProcess = false;

#if 0

-- !loc|objLocTbl|objNum|loc1|loc2|Movable or Fixed  |object|
!000|objLocTbl|61|000|000|Movable|PEARL|


-- !loc|locCondTbl|condition|...|
!001|locCondTbl|HINTF|LIQUID|LIGHT|

#endif

	while ( pDelim != NULL && *(++pDelim) && bContinue )
	{
		if ( *pDelim == delim )
		{
			char* pParam = pDelim+1;

			if ( bObjLocProcess )
			{
				int nVal = *pParam == '-' ? -1 : atoi(pParam);
				switch ( numDelims )
				{
					case 1:	nObj	= nVal; break;
					case 2:	nLoc1	= nVal; break;
					case 3:	nLoc2	= nVal; bContinue = false; break;
				}
				++numDelims;
				continue;
			}
			else if ( bLocCondProcess )
			{
				short nVal = ConvertLocCondStringToShort( pParam );
				nLocCondition += nVal;
				++numDelims;
				continue;
			}
			int nVal = atoi(pParam);

			if ( numDelims == 0 )
			{
				if ( nVal == 0 && strncmp(pParam, "objLocTbl", 9) == 0 )
				{
					nSeq = nVal;
					bObjLocProcess = true;
				}
				else if ( nVal == 0 && strncmp(pParam, "locCondTbl", 10) == 0 )
				{
					nSeq = nVal;
					bLocCondProcess = true;
				}
			}
			++numDelims;
		}
	}

	if ( bObjLocProcess )
	{
		InitPlayObjectLoc(nObj,	nLoc1, nLoc2);
	}
	else if ( bLocCondProcess )
	{
		InitPlayLocationCondtion( nLoc, nLocCondition );
	}

	return bRc;
}


/*
		Routine to process an object.
*/
void CGameState::doobj()
{
	/*
			is he trying to take or drop everything (take all or drop all)?
	*/
	if (e_object == OBJ_ALL) 
	{
		if (e_verb == VRB_DROP)
		{
			if ( m_holding < 1 )
			{
				printf("You have nothing to %s\n", e_word1);
				return;
			}
			for ( e_object = 1; e_object <= LastObjectIndexed(); ++e_object )
			{
				if ( toting (e_object) )
				{
					if ( ! ( e_object == OBJ_WATER || e_object == OBJ_OIL || e_object == OBJ_BIRD ) )
					{
						printf("\t%s\t", vocab(VOCAB_ENTRY_OBJ+e_object));
						trobj();
					}
				}
			}
		}
		else if (e_verb == VRB_TAKE)
		{
			int nObjsFound = 0;
			for ( e_object = 1; e_object <= LastObjectIndexed(); ++e_object )
			{
				if ( m_objectLoc1[e_object] == m_location )
				{
					printf("\t%s\t", vocab(VOCAB_ENTRY_OBJ+e_object));
					trobj();
					++nObjsFound;
				}
			}
			if ( nObjsFound == 0 )
				printf("I see nothing here you can carry.\n");
		}
		else
		{
			printf("Sorry but I don't know how.\n");
		}
		return;
	}	

	/*
			is object here?  if so, transitive
	*/
	if (m_objectLoc2[e_object] == m_location || here(e_object))
		trobj();
	/*
			did he give GRATE as destination?
	*/
	else if (e_object == OBJ_GRATE) 
	{
		if (m_location == LOC_001_end_road || m_location == LOC_004_valley || m_location == LOC_007_slit_streambed) 
		{
			e_motion = MOV_DEPRESSION;
			domove();
		}
		else if (m_location>LOC_009_below_grate && m_location<LOC_015_hall_mists) 
		{
			e_motion = MOV_ENTRANCE;
			domove();
		}
	}
	/*
			is he trying to get at dynamite?
	*/
	else if (e_object == OBJ_ROD && here(OBJ_ROD2)) 
	{
		e_object = OBJ_ROD2;
		trobj();
	}
	/*
			is he trying to get/use a liquid?
	*/
	else if ((liq() == e_object && here(OBJ_BOTTLE)) ||
			liqloc(m_location) == e_object)
			trobj();
	else if (e_object == OBJ_PLANT && at(OBJ_PLANT2) &&
			m_objectProperty[OBJ_PLANT2] == 0) 
	{
		e_object = OBJ_PLANT2;
		trobj();
	}
	/*
			is he trying to grab a knife?
	*/
	else if (e_object == OBJ_KNIFE && m_knifeLoc == m_location) 
	{
		SpeakInfoMsg(MSG4_116_Dwarves_knives_vanish_as_they_strike_walls);
		m_knifeLoc = -1;
	}
	/*
			is he trying to get at shadow troll?
	*/
	else if (e_object == OBJ_TROLL && here(OBJ_TROLL2)) 
	{
		e_object = OBJ_TROLL2;
		trobj();
	}
	/*
			is he trying to get at shadow plant?
	*/
	else if (e_object == OBJ_PLANT && here(OBJ_PLANT2)) 
	{
		e_object = OBJ_PLANT2;
		trobj();
	}
	/*
			is it a DWARF he is after?
	*/
	else if (dwarfCheck() && m_dwarfFlag >= 2) 
	{
		e_object = OBJ_DWARF;
		trobj();
	}
	else
	{
		printf("I see no %s here.\n", probj(e_object));
	}
}

/*
		Routine to process an object being
		referred to.
*/
void CGameState::trobj()
{
	if (e_verb)
		trverb();
	else
		printf("What do you want to do with the %s?\n",	probj(e_object));
}

/*
		Routine to print word corresponding to object
*/
char* CGameState::probj(short object)
{
	short	wtype, wval;
	analyze(e_word1, &wtype, &wval, false);
	return (wtype == VOCAB_TYP_OBJ ? e_word1 : e_word2);
}




/*
		Routine to describe current location
*/
void CGameState::describe( short nType ) //  nType =0, normal; =1, brief; =2, long
{
	if ( ! ValidCaveLocation( m_location ) )
	{
		printf("describe() bad m_location(%d)\n", m_location);
		bug(BUG_PlayGameTurn_Cpp+25);
		return;
	}
	if (toting(OBJ_BEAR))
		SpeakInfoMsg(MSG4_141_followed_by_tame_bear);
	if (dark())
		SpeakInfoMsg(MSG4_016_pitch_dark_If_proceed_likely_fall_into_pit);
	else if ( nType == 2 )
		SpeakLocDescLong(m_location);	// long
	else if ( nType == 1 )
			SpeakLocDescShort(m_location);	// Brief
	else if (m_locVisitedCnt[m_location] && m_brief )
			SpeakLocDescShort(m_location);	// Brief
	else
		SpeakLocDescLong(m_location);	// long
										
	if (m_location == LOC_033_Y2 && pct(25, 12) && !m_closing)
		SpeakInfoMsg(MSG4_008_hollow_voice_says_Plugh);
}

/*
		Routine to describe visible items
*/
void CGameState::descitem()
{
	short	i, state;
	bool	bFirst = true;
										
	if ( ! ValidCaveLocation( m_location ) )
	{
		printf("descitem() bad m_location(%d)\n", m_location);
		bug(BUG_PlayGameTurn_Cpp+27);
		return;
	}

	for ( i = 1; i <= LastObjectIndexed(); ++i ) 
	{
		if (at(i)) 
		{
			if (i == OBJ_STEPS && toting(OBJ_NUGGET))
				continue;
			if (m_objectProperty[i]<0) 
			{
				if (m_closed)
					continue;
				else 
				{
					m_objectProperty[i] = 0;
					if (i == OBJ_RUG || i == OBJ_CHAIN)
						++m_objectProperty[i];
					--m_tally1;
				}
			}
			if (i == OBJ_STEPS && m_location == m_objectLoc2[OBJ_STEPS])
				state = 1;
			else
				state = m_objectProperty[i];
			if ( bFirst )
			{
				puts("\n");
				bFirst = false;
			}
			SpeakObjDesc(i, state);
		}
	}
	if (m_tally1 == m_tally2 && m_tally1 != 0 && m_limit > 35)
		m_limit = 35;
}




/*
		Routine to handle player's demise via
		waking up the DoDwarves...
*/
void CGameState::dwarfend()
{
	death();
	normend();
}

/*
		normal end of game
*/
void CGameState::normend(void)
{
	static short limits[] = 
	{35, 100, 200, 250, 300, 330, 350, 400, 450, 1000};
#define NumLimits (sizeof(limits) / sizeof(limits[0]))
	
	short	idx;
	short	nextLevel;
	short	total = score();
	
	for (idx = 0; idx < NumLimits; idx++)
	{
		if ( total < limits[idx] )
			break;
	}
	puts ("\n");
	SpeakInfoMsg (MSG4_202_Rank_Amateur+idx);
	
	if ( idx < NumLimits - 1 )
	{
		nextLevel = limits[idx] - total;
		printf ("\nTo achieve the next higher rating, you need %d more point%s.\n",	nextLevel, nextLevel > 1 ? "s" : "");
	}
	
	exitAdv(1);
}

/*
		scoring
*/
short CGameState::score()
{
	short s = 0;
	short t = 0;
	short k = 0;
										
	short msg = MSG4_000_No_Message;
										
	for (int ii = FirstTreasure(); ii <= LastTreasure(); ++ii)
	{
		if (ii == OBJ_CHEST)
			k = 14;
		else if (ii > OBJ_CHEST)
				k = 16;
		else 
			k = 12;

		if (m_objectProperty[ii] >= 0)
			t += 2;
#if _DEBUG && 0
		else
			printf("Obj(%d) prop(%d)! >=0\n",ii ,m_objectProperty[ii]);
#endif
		if (m_objectLoc1[ii] == LOC_003_well_house
			&& m_objectProperty[ii] == 0)
			t += k-2;
#if _DEBUG && 0
		else
			printf("Obj(%d) not in well house || prop(%d)!0\n",ii ,m_objectProperty[ii]);
#endif
	}
	// t = 218 is all treasures
	printf("%-20s%d\n", "Treasures:", s = t);
	t = (MAXDIE - m_numdie)*10;
	if (t)
		printf("%-20s%d\n", "Survival:", t);
	s += t;
	if (!m_gaveup)
		s += 4;
	t = m_dwarfFlag > 0 || m_locVisitedCnt[LOC_019_hall_mt_king] > 0 ? 25 : 0;
	if (t)
		printf("%-20s%d\n", "Getting well in:", t);
	s += t;
	t = m_closing ? 25 : 0;
	if (t)
		printf("%-20s%d\n", "Masters section:", t);
	s += t;
	if (m_closed)
	{
		if (m_bonus == MSG4_000_No_Message)	// No explosion
			t = 10;
		else if (m_bonus == MSG4_135_loud_explosion_you_are_splashed_across_walls)
				t = 25;
		else if (m_bonus == MSG4_134_loud_explosion_burying_everything)
				t = 30;
		else if (m_bonus == MSG4_133_loud_explosion_hole_You_can_exit)
				t = 45;
		printf("%-20s%d\n", "Bonus:", t);
		s += t;
	}
	if (m_objectLoc1[OBJ_MAGAZINE] == LOC_108_Witt_s_end)
	{
		s += 1;
	}
										
	t = -15 * m_hintTaken;
	if (t)
	{
		printf("%-22s%4d\n", "Hints & intructions:", t);
		s += t;
	}
	s += 2;

	if ( s >= 350 )
	{
		short nMapLoc = m_objectLoc1[OBJ_MAP];
		short mp = nMapLoc == LOC_143_main_office ? 25 : 0;
		short vp = 0;
		short ws = 0;
		short nLocsVisited = 0;
		short nLocsNotVisited = 0;		// don't count FORCED_MOVE
		for ( short loc = 1;  ValidCaveLocation( loc ); loc++ )
		{
			if ( m_locVisitedCnt[loc] > 0 )
			{
				nLocsVisited++;
			}
			else if ( m_locCondition[loc] != FORCED_MOVE )
			{
				nLocsNotVisited++;
//				printf ("Loc(%d) not visited\n", loc );
			}
		}
		vp = nLocsNotVisited < 10 ? 50 : 0;
		ws = mp + vp > 0 ? 25 : 0;
		if ( ws > 0 )
		{
			printf("%-20s%d\n", "Wizards section:", ws);
			if ( nMapLoc == LOC_143_main_office )
				printf("The Wizard thanks you for returning his map.\n");
			printf("%-20s%d\n", "Bonus of:", vp + mp);
			s += ws + vp + mp;
		}
	}
	if (s < 0)
		s = 0;
	printf("%-22s%d\n", "Score:", s);
	printf("%-22s%d\n", "Turns:", m_turns);
										
	return s;
}

/*
		Routine to handle the passing on of one
		of the player's incarnations...
*/
void CGameState::death()
{
	short	yea, j;

	if ( Get_bNoDieTest() )				// used to test all moves
		return;
	
	if (!m_closing) 
	{
		yea = yes(MSG4_081_killed_might_help_you_out+m_numdie*2, MSG4_082_don_t_blame_me+m_numdie*2, MSG4_054_OK);
		if (++m_numdie >= MAXDIE || !yea)
			normend();
		m_objectLoc1[OBJ_WATER] = LOC_000_Limbo;
		m_objectLoc1[OBJ_OIL] = LOC_000_Limbo;
		if (toting(OBJ_LAMP))
			m_objectProperty[OBJ_LAMP] = 0;
		for ( j = LastObjectIndexed(); j > 0; --j )
		{
			if (toting (j))
				drop(j, j == OBJ_LAMP ? LOC_001_end_road : m_oldloc2 );
		}
		m_newloc1 = LOC_003_well_house;
		m_oldloc1 = m_location;
		return;
	}
	/*
			m_closing -- no resurrection...
	*/
	SpeakInfoMsg(MSG4_131_you_re_dead_it_s_close_closing_time_call_it_a_day);
	++m_numdie;
	normend();
}
